C     Last change:  BG   16 Mar 2006    5:36 pm
      SUBROUTINE A11XST(A11, IER)
C
C
C ======================================================================
C
C                             A 1 1 X S T
C                            =============
C                            =============
C
C     S T A N D            :  01.06.2001
C
C     B E A R B E I T E R  :  DR.-ING. BODO GEIER
C                             DLR BRAUNSCHWEIG, INST. WB-SM
C
C     P U R P O S E :    ADDS CONTRIBUTION OF STIFFENERS RUNNING IN
C     ===============    X-DIRECTION TO MATRIX A11
C
C
C     P A R A M E T E R S :
C     ===================
C
C     INPUT :
C        A11(*)    ARRAY STORING A11
C
C     OUTPUT :
C        A11
C        IER      ERROR INDEX
C                 IER=0:  NO ERROR
C                 IER=1:  FUNCTIONAL DEGREES OF FREEDOM u,v
C                         NOT AVAILABLE
C                 IER=2:  ERROR FROM SUBROUTINE KUORKV
C
C     S U B R O U T I N E S CALLED
C     =============================
C
C      KUORKV     DETERMINES DEGREES OF FREEDOM AT NODES OF
C                 A STIFFENER SECTION
C
C ======================================================================
#ifndef MBEOS5
     INCLUDE '#PRAMS'
#endif
C
C        F07INA
C        ======
      INTEGER :: MX, MY, IBDX, IBDY, KNFUN
      INTEGER, DIMENSION (1:5)::KU
      INTEGER, DIMENSION(1:P_RTNDS,1:P_RTNDS):: LEIT
      REAL, DIMENSION(1:P_RTNDS-1):: A, B
      COMMON /F07INA/ KU,MX,MY,A,B,LEIT,IBDX,IBDY,KNFUN
C
C        F07PD
C        =====
      REAL :: SA, SB, FA, FS
      REAL, DIMENSION(1:4,1:4):: B1
      COMMON /F07PD/  SA,SB,FA,B1,FS
C
C        F07DIS
C        ======
      REAL, DIMENSION(1:4,1:P_STF):: STEIFX, STEIFY
      INTEGER, DIMENSION(1:P_STF):: ISTX, ISTY
      INTEGER :: NOX, NOY
      REAL, DIMENSION(1:P_STF):: DOMX, DOMY
      COMMON /F07DIS/ STEIFX,ISTX,NOX,DOMX, STEIFY,ISTY,NOY,DOMY
C
C        COMLV
C        =====
      INTEGER, DIMENSION(1:P_NX):: LVTRI,LVB
      INTEGER, DIMENSION(1:P_NY):: LVA22
      COMMON /COMLV/  LVTRI,LVA22,LVB
C
C        F01EPS
C        ======
      REAL, DIMENSION(1:11,1:3):: FEPS
      INTEGER, DIMENSION(1:11,1:3):: KEPS
      COMMON /F01EPS/ FEPS, KEPS
C
C        F07IN1,  F07IN2
C        ===============
C     INTEGRALS OVER POLYNOMIAL PRODUCTS * (1-ZETA)
      REAL, DIMENSION(1:3,1:4,1:3,1:4):: S1, S2
      COMMON /F07IN1/ S1
C     INTEGRALS OVER POLYNOMIAL PRODUCTS * ZETA
      COMMON /F07IN2/ S2
C
C        DATRAN
C        ======
      INTEGER:: NREAD, NWRITE, NSTORE
      COMMON /DATRAN/ NREAD,NWRITE,NSTORE
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      DOUBLE PRECISION, DIMENSION(*):: A11
C
      DOUBLE PRECISION ::					F1,F2,F3,F4
      REAL, DIMENSION(1:2,1:4):: 		BX
      REAL, DIMENSION(1:5):: 				FQ
      INTEGER, DIMENSION(1:2,1:5):: 	KQX, KQY
      INTEGER, DIMENSION(1:2,1:4):: 	KPQ
      INTEGER::          					RHO
C
      DATA             KQX/0,0, 0,2, 1,1, 1,2, 2,2/,
     *                 KQY/0,0, 0,0, 1,1, 1,0, 0,0/
C     THERE ARE FIVE RELEVANT TERMS. KQX AND KQY STORE CODE OF
C     DERIVATIVES. FQ WILL TAKE THE FACTORS ACCORDING TO STIFFNESS AND
C     CURVATURE.
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
      IER = 0
C
C     Check functional degrees of freeedom
      IF (.NOT.(KU(2).EQ.2 .AND. KU(3).EQ.3)) THEN
      	WRITE (NWRITE,2000)
         WRITE (NWRITE,2003)
         IER = 1
         GOTO 999
      END IF
C
C     LOOP OVER STIFFENERS
C
      DO 800, I=1,NOX
         III = ISTX(I)
C
C        COMPUTE STIFFNESS FACTORS. CURVATURES ARE TAKEN FROM
C        COMMON /F01EPS /FEPS(*,*).
C		   USE FA = COS(ALFAX-ALFAY), FS = SIN(ALFAX-ALFAY)
         FQ(1) = STEIFX(1,I)*FEPS(1,2)**2
         FQ(2) = 2.*FEPS(1,2)*STEIFX(2,I)
         FQ(3) = STEIFX(4,I)/FA**2
         FQ(4) = -2.*STEIFX(4,I)*FS/FA**2
         FQ(5) = STEIFX(3,I) + STEIFX(4,I)*(FS/FA)**2
C
C        LOOP OVER TERMS
         DO 600, ITRM=1,5
            NUE = KQX(1,ITRM)
            RHO = KQX(2,ITRM)
            MUE = KQY(1,ITRM)
            LAM = KQY(2,ITRM)
            IXP = NUE + RHO - 1
C           FOR USE AS MATRIX SUBSCRIPTS NUE...LAM ARE INCREMENTED BY 1
            NUE = NUE + 1
            RHO = RHO + 1
            MUE = MUE + 1
            LAM = LAM + 1
C
            F1 = FQ(ITRM)
            IF (ABS(F1) .LT. 1.D-30) GOTO 600
C
C           LOOP OVER SUBDIVISIONS
            DO 500, II = 1,MX-1
               RA = A(II)
               F2 = F1/RA**IXP
C              TO ESTABLISH MATRIX BX(2,4) USE MATRIX B1 (COMMON/F07PD)
               BX(1,1) = B1(1,1)
               BX(1,2) = RA*B1(1,2)
               BX(2,1) = B1(2,1)
               BX(2,2) = RA*B1(2,2)
               BX(1,3) = BX(1,1)
               BX(1,4) = BX(1,2)
               BX(2,3) = BX(2,1)
               BX(2,4) = BX(2,2)
               CALL KUORKV('X',III,II,1,KPQ, IER)
                  IF (IER .NE. 0) THEN
                     WRITE (NWRITE,2000)
                     WRITE (NWRITE,2001) II, III
                     IER=2
                     GOTO 999
                  END IF
C              INTERPOLATION LOOPS
               DO 400, IS=1,4
                  JLS = KPQ(LAM,IS)
                  IF (JLS .EQ. 0) GOTO 400
                  F3 = F2*BX(LAM,IS)
                  DO 300, IQ=1,4
                     KMQ = KPQ(MUE,IQ)
                     IF (KMQ .EQ. 0) GOTO 300
                     F4  =
     *               F3*BX(MUE,IQ)*(S1(NUE,IQ,RHO,IS)+S2(NUE,IQ,RHO,IS))
C                    UPDATE MATRIX
                     IF (JLS .GE. KMQ) THEN
                        IA11 = LVTRI(JLS) + KMQ
                     ELSE
                        IA11 = LVTRI(KMQ) + JLS
                     END IF
                     IF (JLS .EQ. KMQ) THEN
                        A11(IA11) = A11(IA11) + F4
                     ELSE
                        A11(IA11) = A11(IA11) + 0.5D0*F4
                     END IF
  300             CONTINUE
  400          CONTINUE
  500       CONTINUE
  600    CONTINUE
  800 CONTINUE
C
  999 RETURN
 2000 FORMAT(/1X,'MESSAGE FROM SUBROUTINE A11XST:')
 2001 FORMAT(1X,'ERROR RETURN FROM KUORKV, I=',I4, 'J=',I4)
 2003 FORMAT(1X,'Using BEOS with stiffened panels requires functional
     * dofs u,v (2,3).'/
     *1X,'Please correct input of KU in NAMELIST SUBDIV')
C     END OF SUBROUTINE A11XST
      END
