C     Last change:  BG    8 Nov 2004   12:49 pm
      SUBROUTINE ABDETC(NLA,THKPLY,ANGPLY,Q,
     *                  THKLAM,AAL,DDL,ADL, IER)
C --------------------------------------------------------------------+
C
C     CONTENTS:  COMPUTE THE LAMINATE STIFFNESS MATRICES A B D H
C
C     INPUT:
C     NLA        Number of plies
C     THKPLY(I)  THICKNESS OF EACH PLY (I .LE. MAXPLY)
C     ANGPLY(I)  PLY ORIENTATION ANGLES (I .LE. MAXPLY).
C                POSITIVE ANGLE MEANS COUNTER-CLOCKWISE ROTATION OF PLY.
C     Q(4)      In-plane material properties of plies: E1,E2,Nue12,G12
C     G(2)      Transverse shear moduli of ply material
C     RETURN:
C     THKLAM    Thickness of laminate
C     AAL(3,3)  LAMINATE MEMBRANE STIFFNESS MATRIX
C     ADL(3,3)  LAMINATE MEMBRANE-BENDING COUPLING STIFFNESS MATRIX
C     DDL(3,3)  LAMINATE BENDING STIFFNESS MATRIX
C     HHL(2,2)  LAMINATE TRANSV.SHEAR STIFFNESS MATRIX
C     DTI(3,3)  INVARIANT BENDING STIFFNESS
C
C      IER      ERROR INDEX
C               IER = 0: NO ERROR ENCOUNTERED
C               IER =21: CALL TO TRNPSI WITH WRONG FIRST ARGUMENT
C               IER =22: RETURN FROM TRNPSI: TRANSVERSE SHEAR STIFFNESS
C                        COULD NOT BE TRANSFORMED
C               IER =23: RETURN FROM TRNPSI: A (3,3)-STIFFNESS MATRIX
C                        COULD NOT BE TRANSFORMED
C               IER =41: MEMBRANE STIFFNESS COULD NOT BE INVERTED
C               IER =42: INVARIANT BENDING STIFFNESS (D TILDE)
C                        COULD NOT BE INVERTED
C               IER =43: TRANSVERSE SHEAR COMPLIANCE
C                        COULD NOT BE INVERTED
CC               IER =51: ERROR IN MATRIX MULTIPLICATION SUBROUTINE
CC                        MULTI2 OR MULTI3 IN THE BRANCH COMPUTING
CC                        THE TRANSVERSE SHEAR STIFFNESS
C
C     COMMON:  ---------------------
C
C
CC     CALLS:    TRNPSI,INVERS,MULTI2,MULTI3
C     CALLS:    TRNPSI,INVERS
C

C     AUTHOR:
C       DR.B.GEIER
C       Equations derived in
C       "Composite Laminate Stffnesses and their Sensitivities"
C        by B. Geier
C       DLR - Institut fuer Strukturmechanik, Braunschweig, Germany,
C       DLR-FB 92-16
C --------------------------------------------------------------------+
C
C      IMPLICIT REAL (A-H, O-Z)
      IMPLICIT NONE
      REAL:: THKLAM
      INTEGER:: NLA,IER
      REAL,DIMENSION(*):: THKPLY,ANGPLY
      REAL,DIMENSION(1:4):: Q
      REAL,DIMENSION(1:3):: ZETA
      REAL,DIMENSION(1:3,1:3):: AAL,ADL,DDL,QR,QL
C
      INTEGER:: NUMPLY,K,I,J
      REAL:: DENO,PSI,S1
      REAL,DIMENSION(1:NLA+1):: Z
C      DIMENSION THKPLY(*),ANGPLY(*), Q(4), G(2)
C      DIMENSION AAL(3,3),ADL(3,3),DDL(3,3),HHL(2,2)
C
C      DIMENSION QR(3,3),GR(2,2),QL(3,3),GL(2,2),Z(NLA+1),
C     *          QLK(3,3,NLA)
C      DIMENSION AH(3,3,NLA),BH(3,3,NLA)
C      DIMENSION AIN(3,3),AB(3,3),BAB(3,3),EM(3,3),YM(3,3),
C     *          DTI(3,3),HINV(2,2)
C      DIMENSION AXL(3,3),X1(3),IP1(3),FA(3,3),
C     *          FP(3,3),FS(3,3),FZ(3,3),
C     *          F0(2,2),F1(2,2),F2(2,2),F3(2,2),
C     *          FG(2,2),FSM(2,2),ZETA(3)
C######################################################################
      NUMPLY = NLA
C######################################################################
      IER=0
C
C     LAMINATE THICKNESS
      THKLAM = THKPLY(1)
      DO, K=2,NUMPLY
         THKLAM = THKLAM + THKPLY(K)
      END DO
C     Z-COORDINATES
      Z(1) =-.5*THKLAM
      DO, K=2,NUMPLY+1
         Z(K) = Z(K-1) + THKPLY(K-1)
      END DO
C
C     MATRIX INITIALIZATIONS
C
      AAL = 0.0
      ADL = 0.0
      DDL = 0.0
C     Transform engineering constants into stiffness
      DENO=1./(1.-Q(3)**2*Q(2)/Q(1))
      QR(1,1) = Q(1)*DENO;  QR(2,2) = Q(2)*DENO; QR(3,3) = Q(4)
      QR(1,2) = Q(3)*QR(2,2); QR(1,3)= 0.; QR(2,3) = 0.
      QR(2,1) = QR(1,2); QR(3,1) = 0.; QR(3,2) = 0.
C
C     COMPUTE AAL,ADL,DDL
C
      ZETA(1) = 1.0
C      DO 200,K=1,NUMPLY
      DO,   K=1,NUMPLY
C
C        STIFFNESS TRANSFORMATION FROM LAYER TO ELEMENT SYSTEM
         PSI= ANGPLY(K)
C
         IF (PSI .GT. 90.0) PSI = PSI - 180
         IF (PSI .LT.-90.0) PSI = PSI + 180
C
C         CALL TRAPSI(3,PSI,QR,QL,IER)
         CALL TRNPSI(3,PSI,QR,QL,IER)
         IF (IER.NE.0) THEN
              IER=20+IER
              GOTO 9999
         ENDIF
C
C        CALCULATE THICKNESS PARAMETERS
C
         DO, I=2,3
            ZETA(I) = Z(K+1)*ZETA(I-1) + Z(K)**(I-1)
         END DO
         DO, I=2,3
            ZETA(I) = ZETA(I)/I
         END DO
C
C        UPDATE LAMINATE STIFFNESS
C
         DO, I=1,3
         DO, J=1,3
            S1 = QL(I,J)*THKPLY(K)
            AAL(I,J)= AAL(I,J) + S1
            ADL(I,J)= ADL(I,J) - S1*ZETA(2)
            DDL(I,J)= DDL(I,J) + S1*ZETA(3)
         END DO
         END DO
      END DO
C
C***  COMPUTE TRANSVERSE SHEAR STIFFNESS (DELETED)
 9999 CONTINUE
C9999  WRITE(*,1000) IER
9090  RETURN
C1000  FORMAT(2x,'ERROR CODE NUMBER = ',I4,
C     *' IN SUBROUTINE COMPUTING COMPOSITE LAMINATE STIFFNESS')
C     END OF SUBROUTINE ABDETC
      END

