C     Last change:  CS   24 Jan 2007   11:02 am
C                                                                               
C     ##################################################################
C                                                                               
C                      PROGRAM   B E O S 4.1_2 (INCORE)
C                      ################################
C                      ###############################
C                      COPYRIGHT BY DLR 2004
C                                                                               
C     S T A T E    :  24.01.2007
C                                                                               
C     A U T H O R  :  Dr.-Ing. B. Geier, DLR, WB-SM
C                                                                               
C     V E R S I O N   F O R   FORTRAN95, Lahey-Fujitsu Compiler
C
C     P U R P O S E :         MAIN PROGRAM
C     ===============
C                                                                               
C     C O M M O N B L O C K S :                                                 
C     =========================                                                 
C                                                                               
C     INPUT   :                                                                 
C     /TITLE/
C       TIT     80 CHARACTERS ANALYSIS HEADLINE
C     /ANTYPE/
C       BUCK      BUCKLING ANALYSIS SWITCH
C       VIBR      NATURAL VIBRATION ANALYSIS SWITCH
C     /COMLV/
C       LV        INDEX VECTOR FOR STORING UPPER TRIANGLES IN A VECTOR
C     /PLOTIN/
C      INDPLT     PLOT INFORMATION OUTPUT SWITCH
C
C     OUTPUT  :
C     /TITLE/
C       TIT     80 CHARACTERS ANALYSIS HEADLINE
C                                                                               
C     S U B P R O G R A M S :                                                   
C     =======================                                                   
C
C     INITLZ      INITIALISE VARIABLES
C     MKLVBD      Make VECTOR LV FOR ADDRESSING BAND MATRICES
C     RDTYPE      READS TITLE LINE AND ANALYSIS TYPE
C     CKTYPE      CHECKS COMPATIBILITY OF ANALYSIS TYPES
C     PRTYPE      PRINTS TITLE LINE AND ANALYSIS TYPE
C     BEBUCK      DETERMINES BUCKLING LOAD FACTORS AND BUCKLING MODES
C     BEVIBR      DETERMINES NATURAL FREQUENCIES AND MODES
C     PLTMSH      ESTABLISH NODE COORDINATES FOR PLOTS
C     CNTOUR      COMPUTE AND PRINT Z-COORDINATES OF PANEL SHAPE
C     MODES       COMPUTE AND PRINT MODE SHAPE INFORMATION
C                                                                               
C ######################################################################        
C
C PARAMETERS USED IN PROGRAM BEOS:
C	==========
C
C	P_RTNDS		Square root of max number of nodes; also max number
C              of nodal lines in either direction.
C
C	P_NODES		P_NODES=P_RTNDS*P_RTNDS 
C     			Max number of nodes
C
C  P_STF        P_STF=P_RTNDS/2
C              Max number of stiffeners in either direction
C
C  P_NSUP       P_NSUP=250
C              Max number of dofs that may be suppressed
C
C	P_NX	   	P_NX = 4*P_NODES
C		      	Max number of dofs from normal displacement w
C
C	P_NY		   P_NY = 16*P_NODES
C		      	Max number of dofs from non-w functional degrees
C		      	of freedom u,v,beta,gamma
C
C	P_IBDX 		P_IBDX = 12*(P_RTNDS+2)
C			      Max bandwidth of geometric stiffness matrix and mass matrix
C
C	P_IBDY	   	P_IBDY = 4*P_IBDX
C			      Max bandwidth of stiffness matrix A22
C
C	P_LA11	   	P_LA11 = P_NX*(P_NX+1)/2
C		      	Max Size of stiffness matrix A11
C              	(Stored as one-dimensional array)
C
C	P_LA22	      P_LA22 = P_NY*P_IBDY - P_IBDY*(IBDY-1)/2
C		      	Max size of stiffness matrix A22
C              	(Stored as one-dimensional array)
C
C  P_LB11		   P_LB11 = P_NX*P_IBDX - P_IBDX*(IBDX-1)/2
C			      Max size of mass matrix and geometric stiffness matrix
C		      	   (Stored as one-dimensional arrays)
C
C	P_NGIT	   	Max number of mode shapes included in iterations
C
C	P_NGRQ       Max number of eigenvalues and mode shapes requested
C
C
C
C	Array Dimensions
C	=================
C
C	X1(P_NX,P_NGRQ)	Mode shapes, transverse deflection (w) part
C
C	X2(P_NY,P_NGRQ)	Mode shapes, non-w part
C
C	LVTRI(P_NX)     Auxiliary vector for index calculation in matrix A11
C                	   (A11 stored as upper triangle)
C
C	LVA22(P_NY)    	Auxiliary vector for index calculation in matrix A22
C                 	(A22 stored as upper half band)
C
C	LVB11(P_NX)     Auxiliary vextor for index calculation in
C			         geometric stiffness and mass matrix
C
C  A(P_RTNDS-1)    ELEMENT (OR SUBDIVISION) SIZES IN X_DIRECTION
C
C  B(P_RTNDS-1)    ELEMENT (OR SUBDIVISION) SIZES IN Y_DIRECTION
C
C  LEIT(P_RTNDS,P_RTNDS)  MATRIX TO BE FILLED WITH NUMBERS OF NODE POINTS
C

C
      SUBROUTINE BEOSMAIN(PROJ,TITP,BUCKP,ITABP,NEIGITP,NEIGRQP,MEMPRIP,
     &   INDPLTP,KUP,AP,BP,BNDX0P,BNDXLP,BNDY0P,BNDYLP,RXP,RYP,RXYP,
     &   ISTXP,ISTYP,ELMSTFP,STEIFXP,STEIFYP,ALFAXP,ALFAYP)
C
#ifndef MBEOS5
     INCLUDE '#PRAMS'
#endif
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER (LEN=80):: TIT
      COMMON /TITLE/  TIT
C
      CHARACTER (LEN=80):: TITP
      LOGICAL BUCKP
C
      LOGICAL MEMPRIP
      INTEGER ITABP,NEIGITP,NEIGRQP,INDPLTP
C      
      INTEGER, DIMENSION (1:5)::KUP
      REAL, DIMENSION (1:P_RTNDS-1):: AP, BP
C      
      INTEGER, DIMENSION (1:7):: BNDX0P,BNDXLP,BNDY0P,BNDYLP
C      
      REAL :: RXP,RYP,RXYP
      INTEGER, DIMENSION(1:P_STF)::ISTXP,ISTYP
      LOGICAL TEMP,STRS,BUCK,VIBR,INPB
      COMMON /ANTYPE/ TEMP,STRS,BUCK,VIBR,INPB,CRIPAN,GLBAN
C
      INTEGER, DIMENSION(1:P_NX):: LVTRI,LVB
      INTEGER, DIMENSION(1:P_NY):: LVA22
      COMMON /COMLV/  LVTRI,LVA22,LVB
C
	  REAL :: ALFAXP,ALFAYP
C
C         / M A N /
C         =========
      INTEGER :: IX,IY,ITER,NEIGIT,ITAB,NEIGRQ
      LOGICAL MEMPRI,X2PRI
      COMMON /MAN/ IX,IY,ITER,NEIGIT,ITAB,NEIGRQ,MEMPRI,X2PRI
C
C        / F 0 7 I N A /
C        ===============
      INTEGER :: MX, MY, IBDX, IBDY, KNFUN
      INTEGER, DIMENSION (1:5)::KU
      INTEGER, DIMENSION(1:P_RTNDS,1:P_RTNDS):: LEIT
      REAL, DIMENSION(1:P_RTNDS-1):: A, B
      COMMON /F07INA/ KU,MX,MY,A,B,LEIT,IBDX,IBDY,KNFUN
C
C       / P L O T I N /
C      ===============
      REAL:: SCAL
      INTEGER :: INDPLT,NPX,NPY,ISYMX,ISYMY,NMOD
      INTEGER,DIMENSION(1:P_NGRQ):: MDSPLT
      COMMON /PLOTIN/ SCAL,INDPLT,NPX,NPY,ISYMX,ISYMY,MDSPLT,NMOD
C
C        / F 0 7 I N D /
C       ===============
      LOGICAL SEITEN
      INTEGER :: NUMSUP
      INTEGER, DIMENSION(1:7):: BNDY0,BNDX0,BNDYL,BNDXL
      INTEGER, DIMENSION(1:4,1:P_NSUP):: IDESUP
      COMMON /F07IND/ SEITEN,BNDY0,BNDX0,BNDYL,BNDXL,IDESUP,NUMSUP
C
C
C         / F 0 7 D I S /
C         ===============
	  REAL, DIMENSION (1:4,P_STF):: STEIFXP,STEIFYP
      REAL, DIMENSION(1:4,1:P_STF):: STEIFX, STEIFY
      INTEGER, DIMENSION(1:P_STF):: ISTX, ISTY
      INTEGER :: NOX, NOY,STYPE
      REAL, DIMENSION(1:P_STF):: DOMX, DOMY
      COMMON /F07DIS/ STEIFX,ISTX,NOX,DOMX, STEIFY,ISTY,NOY,DOMY,STYPE
      COMMON /DATRAN/ NREAD,NWRITE,NSTORE
C
C        / F 0 7 I N B /
C        ===============
      REAL :: RX, RY, RXY
      COMMON /F07INB/ RX,RY,RXY
C
C         / F 0 7 A N G /
C         ===============
      REAL :: ALFAX, ALFAY
      COMMON /F07ANG/ ALFAX,ALFAY
C
C     S T I F F N E S S  D I S T R I B U T I O N
C     ===========================================
	  INTEGER,DIMENSION(1:P_RTNDS-1) :: ELMSTFP,ELMSTF
	  COMMON /STIFFCOM/ ELMSTF
C
	  REAL, DIMENSION (P_NODES)::ARLDVX,ARLDVY,ARLDVXY
      LOGICAL::      OP25
      CHARACTER*8::  PROJ
C
C
      CALL INITLZ
C     Open Input/Output files
C
      TIT=TITP
      BUCK=BUCKP
C     Namelist CONST
      ITAB=ITABP
      NEIGIT=NEIGITP
      NEIGRQ=NEIGRQP
      MEMPRI=MEMPRIP
      INDPLT=INDPLTP      
C     Namelist SUBDIV
      KU=KUP
      A=AP
      B=BP
C     Namelist BNDCON
      BNDX0=BNDX0P
      BNDXL=BNDXLP
      BNDY0=BNDY0P
      BNDYL=BNDYLP      
C     COUNT NUMBER OF FUNCTIONAL DOFS
      KNFUN = 0
      DO, I=1,5
         IF ( KU(I).GT.0 )   KNFUN = KNFUN + 1
      END DO
C     REARRANGE KU(I), DENSE IN THE LEADING PLACES
      JE = 5
      DO, I=1,KNFUN
  240    CONTINUE
         IF ( KU(I).NE.0 ) CYCLE
         JA = I + 1
         DO, J=JA,JE
            KU(J-1) = KU(J)
         END DO
         KU(JE) = 0
         JE = JE - 1
         GOTO 240
      END DO
C     SORT KU(I) IN INCREASING ORDER
      IF (.NOT.( KNFUN.EQ.1 )) THEN
         KM = KNFUN - 1
         OUTER:  DO, I=1,KM
            K = KU(I)
            JC = 0
            IP = I + 1
            INNER:  DO, J=IP,KNFUN
               L = KU(J)
               IF ( L.GE.K ) CYCLE INNER
               JC = J
               K = L
            END DO INNER
            IF ( JC.EQ.0 ) CYCLE OUTER
            KC = KU(JC)
            KU(JC) = KU(I)
            KU(I) = KC
         END DO OUTER
      END IF
C	  Namelist RAD
      IF (.NOT.(KNFUN.EQ.1.OR.( KNFUN.EQ.3 .AND. KU(2).EQ.4))) THEN
C     *    READ (NREAD,RAD,END=104)
		RX=RXP
		RY=RYP
		RXY=RXYP
	  END IF
C     Namelist STIFF
      ISTX(1:P_STF)=ISTXP(1:P_STF)
	  ELMSTF(1:P_RTNDS-1)=ELMSTFP(1:P_RTNDS-1)
	  STEIFX=STEIFXP
	  ISTY(1:P_STF)=ISTYP(1:P_STF)
      STEIFY=STEIFYP
C	  Namelist ANGL
	  ALFAX=ALFAXP
	  ALFAY=ALFAYP
C      WRITE (*,'(" Enter project name: It will be used to form names",
C     *" of input and output File.",/
C     * "Enter name now (up to 8 characters:")')
C      READ (*,'(A8)',ERR=1001) PROJ
C      WRITE (*,'("Input file is read")')
      NREAD = 8
C      OPEN (NREAD, FILE=PROJ//'.inp')
      NWRITE = 9
      OPEN (NWRITE, FILE=PROJ//'.out')
C      NSTORE = 11
C      OPEN (NSTORE, File = PROJ//'_ST.ORE',STATUS='SCRATCH')
C
C     P R E L I M I N A R Y   COMPUTATIONS                                      
C
C     Establish array LV (for matrices stored as full triangles)
      IERR = 0
      CALL MKLVBD(LVTRI,P_NX,P_NX, IERR)
         IF (IERR .NE. 0) THEN
            WRITE (NWRITE,2001)
            WRITE (*,2001)
            WRITE (NWRITE,2002)
            WRITE (*,2002)
            GOTO 9999
         END IF
C                                                                       
C          READ TITLE LINE AND ANALYSIS TYPE
C      CALL RDTYPE(IERR)
C         IF ((IERR.NE.0)) THEN
C            WRITE (NWRITE,2001)
C            WRITE (*,2001)
C            WRITE (NWRITE,2003)
c            WRITE (*,2003)
C            GOTO 9999
C         END IF
C
C          CHECK COMPATIBILITY OF ANALYSIS TYPES
      CALL CKTYPE(IERR)
         IF (IERR .NE. 0) THEN
            WRITE (NWRITE,2001)
            WRITE (*,2001)
            WRITE (NWRITE,2004)
            WRITE (*,2004)
            GOTO 9999
         END IF
C
C          PRINT TITLE LINE AND ANALYSIS TYPE
      CALL PRTYPE
C
C     A N A L Y S I S   T Y P E S
C
C          BUCKLING LOAD FACTORS AND MODES
C      IF (BUCK) THEN
         PRINT*,' Analysis started'
         CALL BUVI(IERR)
            IF (IERR.NE.0) THEN
               WRITE (NWRITE,2001)
               WRITE (*,2001)
               WRITE (NWRITE,2005)
               WRITE (*,2005)
               GOTO 9999
            END IF
C
         IF (INDPLT .NE. 0) THEN
            OPEN(26, FILE = PROJ//'MODES.txt')
            REWIND 26
            CALL PLTMSH
            CALL CNTOUR
            IER=0
            CALL MODES(IER)
            CLOSE (26,STATUS='KEEP')
            IF (IER.NE.0) THEN
               WRITE (NWRITE,2008)
               WRITE (*,2008)
               GOTO 9999
            END IF
         END IF
C      END IF

C          NATURAL FREQUENCIES AND MODES
C      IF (VIBR) THEN
C         PRINT*,' Vibration analysis started'
C        CALL BEVIBR(IERR)
C            IF (IERR.NE.0) THEN
C               WRITE (NWRITE,2001)
C               WRITE (*,2001)
C               WRITE (NWRITE,2007)
C               WRITE (*,2007)
C               GOTO 9999
C            END IF
C
C         IF (INDPLT .NE. 0) THEN
C            OPEN(26, FILE = PROJ//'VIBMODES.txt')
C            REWIND 26
C            IF (.NOT.BUCK)  CALL PLTMSH
C            CALL CNTOUR
C            IER=0
C            CALL MODES(IER)
C            CLOSE (26,STATUS='KEEP')
C            IF (IER.NE.0) THEN
C               WRITE (NWRITE,2008)
C               WRITE (*,2008)
C               GOTO 9999
C            END IF
C         END IF
C      END IF

      CLOSE (NWRITE,STATUS='KEEP')
      PRINT*,' Buckling/Vibration analysis finished'
C
 9999 INQUIRE (25,OPENED=OP25)
      IF (OP25) CLOSE (25,STATUS='DELETE')
C      GOTO 1002
	  GOTO 2009
C
 1001 WRITE (NWRITE,2006)
      WRITE (*,2006)
C
 1002 STOP
C
C
 2001 FORMAT ( 4X,'MAIN PROGRAM:   EXECUTION STOPPED.')
 2002 FORMAT (16X,'ERROR ENCOUNTERED WHEN TRYING TO MAKE ARRAY LVTRI'/
     &        16X,'IN SUBROUTINE MKLVBD')
 2003 FORMAT (16X,'ERROR WHEN TRYING TO READ ANALYSIS TYPE'/
     &        16X,'IN SUBROUTINE RDTYPE')
 2004 FORMAT (16X,'ERROR ENCOUNTERED WHEN CHECKING ANALYSIS TYPE'/
     &        16X,'IN SUBROUTINE CKTYPE')
 2005 FORMAT (16X,'ERROR REPORT FROM SUBROUTINE BUVI')
C
 2006 FORMAT (16X,'ERROR ENCOUNTERED WHEN READING PROJECT NAME')
C 2007 FORMAT (16X,'ERROR REPORT FROM SUBROUTINE BEVIBR')
 2008 FORMAT (16X,'ERROR WHEN COMPUTING/PRINTING MODE SHAPES')
 2009 END                                                                       
C
#ifndef MBEOS5
     INCLUDE 'BEOSData.for'
     INCLUDE 'BEOSBuck.for'
#endif
C
      SUBROUTINE INITLZ
#ifndef MBEOS5
     INCLUDE '#PRAMS'
#endif

C
      REAL, DIMENSION(1:4,1:P_STF):: STEIFX, STEIFY
      INTEGER, DIMENSION(1:P_STF):: ISTX, ISTY
      INTEGER :: NOX, NOY
      REAL, DIMENSION(1:P_STF):: DOMX, DOMY
      COMMON /F07DIS/ STEIFX,ISTX,NOX,DOMX, STEIFY,ISTY,NOY,DOMY
C
      INTEGER, DIMENSION (1:5)::KU
      INTEGER, DIMENSION(1:P_RTNDS,1:P_RTNDS):: LEIT
      REAL, DIMENSION(1:P_RTNDS-1):: A, B
      COMMON /F07INA/ KU,MX,MY,A,B,LEIT,IBDX,IBDY,KNFUN
C
      LOGICAL SEITEN
      INTEGER, DIMENSION(1:7):: BNDY0,BNDX0,BNDYL,BNDXL
      INTEGER, DIMENSION(1:4,1:P_NSUP):: IDESUP
      COMMON /F07IND/ SEITEN,BNDY0,BNDX0,BNDYL,BNDXL,IDESUP,NUMSUP
C
      REAL, DIMENSION(1:P_NODES,1:3):: PVAR, PFEST
      COMMON /CPVAR/  PVAR
      COMMON /CPFEST/ PFEST
C
      REAL:: DOM, PDOM
      COMMON /DENSIT/ DOM, PDOM
C
      DO, I=1,P_STF
         DO, J=1,4
            STEIFX(J,I) = 0.0; STEIFY(J,I) = 0.0
         END DO
         ISTX(I)=0; ISTY(I)=0
      END DO
      DOM= 1.0E-6; DOMX = 0.0; DOMY = 0.0
      DO, I=1,P_RTNDS-1
         A(I)=0. ; B(I)=0.
      END DO
      DO, I=1,P_RTNDS
         DO, J=1,P_RTNDS
            LEIT(I,J) = 0
         END DO
      END DO
      DO, I=1,P_NSUP
         DO, J=1,4
            IDESUP(J,I)=0
         END DO
      END DO
C      DO, I=1,P_NODES
C      DO, J=1,3
C         PVAR(I,J) = 0.
C         PFEST(I,J)= 0.
C      END DO
C      END DO
C      END SUBROUTINE INITLZ
      END
C     Last change:  DBG  13 Dec 2001    3:33 pm



