C     Last change:  BG   24 Oct 2005    4:59 pm
      SUBROUTINE BUVI(IER)
C
C ######################################################################
C
C                               B U V I
C                               #######
C                               #######        COPYRIGHT BY DLR 1991
C
C     S T A T E    :   24.10.2005
C	M O D I F I E D: 11.06.2007 by Conny Schillo
C                                 DLR BRAUNSCHWEIG, INST. FA-SM
C                                 to include crippling analysis
C
C     A U T H O R  :  B. Geier, DLR BRAUNSCHWEIG, INST. WB-SM
C
C
C     P U R P O S E : SETS UP STIFFNESS AND GEOMETRIC STIFFNESS OR MASS
C     =============== MATRIX. SOLVES EIGENVALUE PROBLEM TO DETERMINE
C                     BUCKLING LOADS OR VIBRATION FREQUENCES AND
C                     CORRESPONDING MODES.
C
C
C     I N P U T :
C     ===========
C     BY ARGUMENT:
C        IER      ERROR INDEX. IER=0 MEANS NO ERROR
C     VIA COMMON:
C     /SHLSTF/
C       C(45)     SHELL STIFFNESSES
C                 MEMBRANE, BENDING, BENDING CORRECTION AND COUPLING
C                 UPPER TRI E, ROW-WISE
C       CH(3)     TRANSVERSE SHEAR STIFFNESSES, UPPER TRIANGLE
C
C     S U B P R O G R A M S :
C     =======================
C
C     RDDATA      READS INPUT DATA TO SPECIFY TASK
C     PRTOUT      PRINTS OUT INPUT DATA
C     SETKEY      SETS UP KEY MATRIX SPECIFYING THE DOFS
C     F01TFM      TRANSFORMS CONSTITUTIVE LAW IN CASE OF OBLIQUE AXES
C     F01KBF      FORMS QUADRATIC ENERGY TERMS FROM STRAIN-DISPLACEMENT
C                 RELATIONS
C     REDFKQ      REDUCES LIST OF ENERGY TERMS
C     PLOUT       PUTS OUT PLOT RAW DATA
C
C ######################################################################
C
#ifndef MBEOS5
     INCLUDE '#PRAMS'
#endif
C	==========
C
C	P_RTNDS		Square root of max number of nodes; also max number
C              of nodal lines in either direction.
C
C	P_NODES		P_NODES=P_RTNDS*P_RTNDS 
C     			Max number of nodes
C
C  P_STF        P_STF=P_RTNDS/2
C              Max number of stiffeners in either direction
C
C  P_NSUP       P_NSUP=250
C              Max number of dofs that may be suppressed
C
C	P_NX	   	P_NX = 4*P_NODES
C		      	Max number of dofs from normal displacement w
C
C	P_NY		   P_NY = 16*P_NODES
C		      	Max number of dofs from non-w functional degrees of freedom
C		      	u,v,beta,gamma
C
C	P_IBDX 		P_IBDX = 12*(P_RTNDS+2)
C			      Max bandwidth of geometric stiffness matrix and mass matrix
C
C	P_IBDY	   	P_IBDY = 4*P_IBDX
C			      Max bandwidth of stiffness matrix A22
C
C	P_LA11	   	P_LA11 = P_NX*(P_NX+1)/2
C		      	Max Size of stiffness matrix A11
C              	(Stored as one-dimensional array)
C
C	P_LA22	      P_LA22 = P_NY*P_IBDY - P_IBDY*(IBDY-1)/2
C		      	Max size of stiffness matrix A22
C              	(Stored as one-dimensional array)
C
C  P_LB11		   P_LB11 = P_NX*P_IBDX - P_IBDX*(IBDX-1)/2
C			      Max size of mass matrix and geometric stiffness matrix
C		      	   (Stored as one-dimensional arrays)
C
C	P_NGIT	   	Max number of mode shapes included in iterations
C
C	P_NGRQ       Max number of eigenvalues and mode shapes requested
C
C
C
C	Array Dimensions
C	=================
C
C	X1(P_NX,P_NGRQ)	Mode shapes, transverse deflection (w) part
C
C	X2(P_NY,P_NGRQ)	Mode shapes, non-w part
C
C	LVTRI(P_NX)     Auxiliary vector for index calculation in matrix A11
C                	   (A11 stored as upper triangle)
C
C	LVA22(P_NY)    	Auxiliary vector for index calculation in matrix A22
C                 	(A22 stored as upper half band)
C
C	LVB11(P_NX)     Auxiliary vextor for index calculation in
C			         geometric stiffness and mass matrix
C
C  A(P_RTNDS-1)    ELEMENT (OR SUBDIVISION) SIZES IN X_DIRECTION
C
C  B(P_RTNDS-1)    ELEMENT (OR SUBDIVISION) SIZES IN Y_DIRECTION
C
C  LEIT(P_RTNDS,P_RTNDS)  MATRIX TO BE FILLED WITH NUMBERS OF NODE POINTS
C
C     COMMON blocks
C     *****************************************************************
C
      REAL::FSTBUB
C
C         / M A N /
C         =========
      INTEGER :: IX,IY,ITER,NEIGIT,ITAB,NEIGRQ
      LOGICAL MEMPRI,X2PRI
      COMMON /MAN/ IX,IY,ITER,NEIGIT,ITAB,NEIGRQ,MEMPRI,X2PRI
C
C      / C O M L V /
C      =============
      INTEGER, DIMENSION(1:P_NX):: LVTRI,LVB
      INTEGER, DIMENSION(1:P_NY):: LVA22
      COMMON /COMLV/  LVTRI,LVA22,LVB
C
C      / F 0 7 I N A /
C      ===============
      INTEGER, DIMENSION (1:5)::KU
      INTEGER, DIMENSION(1:P_RTNDS,1:P_RTNDS):: LEIT
      REAL, DIMENSION(1:P_RTNDS-1):: A, B
      COMMON /F07INA/ KU,MX,MY,A,B,LEIT,IBDX,IBDY,KNFUN
C
C      / F 0 7 D I S /
C      ===============
      REAL, DIMENSION(1:4,1:P_STF):: STEIFX, STEIFY
      INTEGER, DIMENSION(1:P_STF):: ISTX, ISTY
      INTEGER :: NOX, NOY,STYPE
      REAL, DIMENSION(1:P_STF):: DOMX, DOMY
      COMMON /F07DIS/ STEIFX,ISTX,NOX,DOMX, STEIFY,ISTY,NOY,DOMY,STYPE
C
C      / F 0 7 A I J /
C      ===============
      REAL, DIMENSION(1:3,1:3):: AIJ
      COMMON /F07AIJ/ AIJ
C
C      / S H L S T F /
C      ===============
      REAL, DIMENSION(45,P_ELMTS):: C
      REAL, DIMENSION(3,P_ELMTS):: CH
C      COMMON /SHLSTF/ C,CH
C
      INTEGER :: IELM
C
C      /  P L O T F L S /
C       ==================
      REAL::                              LX,LY,DX,DY
      REAL,DIMENSION(1:81)::              XPLT,YPLT
      REAL,DIMENSION(1:81,1:81)::         CNTR
      COMMON /PLOTFLS/ XPLT,YPLT,LX,LY,DX,DY,CNTR
C
C       / P L O T I N /
C      ===============
      REAL:: SCAL
      INTEGER :: INDPLT,NPX,NPY,ISYMX,ISYMY,NMOD
      INTEGER,DIMENSION(1:P_NGRQ):: MDSPLT
      COMMON /PLOTIN/ SCAL,INDPLT,NPX,NPY,ISYMX,ISYMY,MDSPLT,NMOD
C
C        / A N T Y P E /
C        ===============
      LOGICAL TEMP,STRS,BUCK,VIBR,INPB,CRIPAN,GLBAN
      COMMON /ANTYPE/ TEMP,STRS,BUCK,VIBR,INPB,CRIPAN,GLBAN
C
C      / D A T R A N /
C      ===============
      INTEGER:: NREAD,NWRITE,NSTORE
      COMMON /DATRAN/ NREAD,NWRITE,NSTORE
C
C        / F 0 7 I N C /
C        ===============
      REAL, DIMENSION(1:6,P_ELMTS) :: RR, DR, BR
      REAL, DIMENSION(1:3,P_ELMTS) :: SS
C     
C        / F 0 7 I N B /
C        ===============
      REAL :: RX, RY, RXY
      REAL, DIMENSION(1:6,P_ELMTS):: BB, DD, BD
      COMMON /F07INB/ RX,RY,RXY
C     *****************************************************************
C
C                  PRINT ANALYSIS TYPE
      WRITE (NWRITE,'(///14X,''BUCKLING LOAD FACTORS AND MODES'')')
C
C     P R E L I M I N A R Y   COMPUTATIONS
C
      IER = 0
      IERR = 0
C      READ INPUT DATA
      CALL RDDATA(BB,DD,BD,RR,DR,BR,SS,IERR)
      IF (IERR.NE.0) THEN
         IER = IERR
         WRITE (NWRITE,2000)
         WRITE (NWRITE,2001)'RDDATA',IER
         WRITE (*,2000)
         WRITE (*,2001)'RDDATA',IER
         GOTO 9999
      END IF
      IF ( MY*MX .GT. P_NODES ) THEN
         WRITE (NWRITE,9510)   P_NODES,MX,MY
         IER = 1
         GOTO 9999
      END IF
C
C     P R I N T O U T   OF INPUT DATA
C
      IF (VIBR) THEN
C         CALL PRTVIB
      ELSE
         CALL PRTOUT (BB,DD,BD,RR,DR,BR,SS) 
      END IF
C     SET UP KEY MATRIX
      CALL SETKEY(IX,IY)
C
C     SET AUXILIATY MATRICES FOR INDEX CALCULATION
      IF (IY .NE. 0) THEN
         CALL MKLVBD(LVA22,IY,IBDY, IERR)
      	IF (IERR .NE. 0) THEN
         	WRITE (NWRITE,2000)
         	WRITE (NWRITE,2001) 'MKLVBD',IERR
         	WRITE (NWRITE,'(" for LVA22")')
         	WRITE (*,2000)
         	WRITE (*,2001) 'MKLVBD',IERR
         	WRITE (*,'(" for LVA22")')
         	IER = IERR
         GOTO 9999
      END IF
      END IF
      CALL MKLVBD(LVB,IX,IBDX, IERR)
      IF (IERR .NE. 0) THEN
         WRITE (NWRITE,2000)
         WRITE (NWRITE,2001) 'MKLVBD',IERR
         WRITE (NWRITE,'(" for LVB")')
         WRITE (*,2000)
         WRITE (*,2001) 'MKLVBD',IERR
         WRITE (*,'(" for LVB")')
         IER = IERR
         GOTO 9999
      END IF
C
C     TRANSFORM TO PANEL COORDINATE SYSTEM
      CALL TFMSTF(C,CH,BB,DD,BD,RR,DR,BR,SS,IERR)
      IF (IERR.NE.0) THEN
         IER = IERR
         WRITE (NWRITE,2000)
         WRITE (NWRITE,2001)'TFMSTF',IERR
         WRITE (*,2000)
         WRITE (*,2001)'TFMSTF',IERR
         GOTO 9999
      END IF
C
C     IF PANEL IS STIFFENED COMPUTE IN_PLANE COMPLIANCE OF SKIN
C********!!!!! E L E M E N T  N O C H  F E S T G E L E G T !!!!!
      IELM =1
      IF ( NOX.NE.0 .OR. NOY.NE.0 )   THEN
         CCC = C(1,IELM)*C(10,IELM)*C(18,IELM)
         HA = C(10,IELM)*C(18,IELM) - C(11,IELM)**2
         HB = C(3,IELM)*C(11,IELM)  - C(2,IELM)*C(18,IELM)
         HC = C(2,IELM)*C(11,IELM)  - C(10,IELM)*C(3,IELM)
         HD = C(2,IELM)*C(3,IELM)   - C(1,IELM)*C(11,IELM)
         DELTA = C(1,IELM)*HA + C(2,IELM)*HB + C(3,IELM)*HC
         IF (CCC.GT.0.1E-30.AND.ABS(DELTA).GT.0.1E-30*CCC) THEN
            DELTA = 1./DELTA
          	AIJ(1,1)=HA; AIJ(1,2)=HB; AIJ(1,3)=HC
          	AIJ(2,2)=(C(1,IELM)*C(18,IELM)-C(3,IELM)**2); AIJ(2,3)=HD
            AIJ(3,3)=C(1,IELM)*C(10,IELM)-C(2,IELM)**2
         	AIJ(2,1)=AIJ(1,2); AIJ(3,1)=AIJ(1,3); AIJ(3,2)=AIJ(2,3)
            AIJ = DELTA*AIJ
C            WRITE (NWRITE,'(/"TEST AIJ")')
C            WRITE (NWRITE,'(3(1X,3E15.6))') ((AIJ(K,L),L=1,3),K=1,3)
         ELSE
            WRITE (NWRITE,9520)
            IER = 10
            GOTO 9999
         END IF
      ENDIF
C
      CALL F01KBF(C, CH)
      CALL REDFKQ
      CALL CHNEIG
C
      IF (BUCK) THEN
C     S E T   U P   M A T R I C E S
         CALL INCS2(IERR)
            IF (IERR .NE. 0) THEN
               WRITE (NWRITE,2000)
               WRITE (NWRITE,2001) 'INCS2'
               WRITE (*,2000)
               WRITE (*,2001) 'INCS2'
               IER = 2
               GOTO 9999
            END IF
C
C
C     C A L C U L A T E   S T R I N G E R   C R I P P L I N G   F O R C E
      IF (CRIPAN) THEN
        SELECT CASE (STYPE)
            CASE (1)
              CALL CRIPPLINGT (IERR)
            CASE (2)
                CALL CRIPPLINGO (IERR)
            CASE DEFAULT
                WRITE (NWRITE,2000)
                WRITE (NWRITE, 2001) 'CRIPPLING'
                IERR=4
                GOTO 9999
        END SELECT
      END IF
C
C     S O L V E   E I G E N V A L U E   P R O B L E M
         CALL INCS3(IY,BB,DD,BD,RR,DR,BR,SS,IERR,FSTBUB)
            IF (IERR .NE. 0) THEN
               WRITE (NWRITE,2000)
               WRITE (NWRITE,2001) 'INCS3'
               WRITE (*,2000)
               WRITE (*,2001) 'INCS3'
               IER = 3
               GOTO 9999
            END IF
      END IF
      IF (VIBR) THEN
        WRITE (NWRITE,*) 'ANALYSIS OF EIGENFREQUENCIES NOT SUPPORTED IN
     * THIS VERSION OF BEOS!'  
      END IF  
C     S E T   U P   M A T R I C E S
C         CALL VIBS2(IERR)
C            IF (IERR .NE. 0) THEN
C               WRITE (NWRITE,2000)
C               WRITE (NWRITE,2001) 'VIBS2'
C               IER = 2
C               GOTO 9999
C            END IF
C
C     S O L V E   E I G E N V A L U E   P R O B L E M
C         CALL VIBS3(IY, IERR)
C            IF (IERR .NE. 0) THEN
C               WRITE (NWRITE,2000)
C               WRITE (NWRITE,2001) 'VIBS3'
C              IER = 3
C               GOTO 9999
C            END IF
C      END IF
C     G L O B A L  B U C K L I N G  A N A L Y S I S
      IF (GLBAN) THEN
        CALL GLBUCK (BB,DD,BD,IERR,FSTBUB)
        IF (IERR.NE.0) THEN
            WRITE (NWRITE,2000)
            WRITE (NWRITE,2001) 'GLBUCK'
            IER=4
            GOTO 9999
        END IF
      END IF
C
C     S A V E   P L O T   D A T A    (IF WANTED)
C      IF ( INDPLT.GT.0 )   CALL PLOUT
C
C
 9999 RETURN
C
 2000 FORMAT (1X,'MESSAGE FROM SUBROUTINE BUVI:')
 2001 FORMAT (1X,'ERROR REPORTED FROM SUBROUTINE ',A10,'  IER=',I3)
 9510 FORMAT (' MX*MY .GT. P_NODES. P_NODES=',I5,'  MX =',I5,' MY =',I5)
 9520 FORMAT (' MESSAGE FROM SUBR. BUVI:'/'MEMBR. STIFFN. INCORRECT')
C     END SUBROUTINE BUVI
      END
C
