C     Last change:  BG   22 Nov 2004   11:33 am
      SUBROUTINE CHODEC(A,N,IBD,LV,IER)
C
C ======================================================================
C
C                               C H O D E C
C                               ===========
C                               ===========
C
C     S T A N D            :  14.01.88
C
C     B E A R B E I T E R  :  DR. GEIER, SCHOUREK, RENKEN
C                             DFVLR BRAUNSCHWEIG, INST. WB-SM
C
C     Z W E C K : CHOLESKY-ZERLEGUNG DER BANDMATRIX A IN R(TRANSP)*R
C     =========== A IST LINEAR GESPEICHERT UND WIRD DURCH R UEBER-
C                 SCHRIEBEN.
C
C
C
C     P A R A M E T E R : (REIHENFOLGE NACH DER PARAMETERLISTE)
C     ===================
C
C     EINGABE :
C     A....... (REAL*8)  SYM. (N,N)-BANDMATRIX, LINEAR GESPEICHERT,
C                        S. HINWEISE
C     N....... (INTEGER) ANZAHL DER ZEILEN DER MATRIX A
C     IBD..... (INTEGER) BANDBREITE DER MATRIX A, IBD < N + 1
C                        (ANZAHL DER NEBENDIAGONALEN EINES DREIECKS + 1
C                        (HAUPTDIAGONALE))
C     LV...... (INTEGER) VEKTOR DER LAENGE N, ZUR INDEXBERECHNUNG DER
C                        MATRIX A, S. HINWEISE
C
C     AUSGABE :
C     A....... (REAL*8)  CHOLESKY-ZERLEGTE R DER MATRIX A,
C                        BANDSTRUKTURIERTE OBERE DREIECKSMATRIX DER
C                        DIMENSION (N,N), BANDBREITE IBD, S. HINWEISE
C     IER..... (INTEGER) FEHLERPARAMETER
C                        IER = 0: ZERLEGUNG FEHLERFREI
C                        IER = I: MATRIX A NICHT POSITIV DEFINIT;
C                            ELEMENT, DAS ZUM FEHLER FUEHRTE, IST A(I)
C
C     H I N W E I S E :
C     =================
C
C     DIE SYMMETRISCHE MATRIX A IST LINEAR SPALTENWEISE GESPEICHERT,
C     UND ZWAR NUR DAS BAND IM OBEREN DREIECK.
C     INDEXBERECHNUNG: ELEMENT (I,J) IST A(LV(J)+I)
C     DIMENSION BEI IBD = N: N*(N+1)/2
C               BEI IBD < N: N*IBD - (IBD*(IBD-1)/2)
C
C     DIE OBERE DREIECKSMATRIX R IST LINEAR SPALTENWEISE GESPEICHERT,
C     UND ZWAR NUR DAS BAND.
C     INDEXBERECHNUNG: ELEMENT (I,J) IST A(LV(J)+I)
C
C ======================================================================
C
      DOUBLE PRECISION A(1), H1, H2, DSQRT
      DIMENSION        LV(N)
      LOGICAL          BL
C
      IER = 0
      IF (A(1) .LE. 1.D-20)   GOTO 60
      H1   = DSQRT(A(1))
      A(1) = H1
      IF (N .EQ. 1) GOTO 60
      DO 10, J=2,IBD
         JL    = LV(J) + 1
         A(JL) = A(JL) / H1
   10 CONTINUE
      I     = 2
   20 LI    = LV(I)
      JA    = MAX0(1,I-IBD+1)
      JANZ  = I - JA
      II    = I + LI
      JA    = JA + LI
      JEE   = JA + JANZ -1
C      H1    = A(II) - QVSKPX(A(JA),A(JA),JANZ,1,1)
C      H1    = A(II) - DOT_PRODUCT(A(JA:JEE),A(JA:JEE))
C      WRITE(*,*)A(II)
      H1    = A(II) -
     *        DOT_PRODUCT(A((/(K,K=JA,JEE)/)),A((/(K,K=JA,JEE)/)))
      IF (H1 .LE. 1.D-20)   GOTO 70
      H1    = DSQRT(H1)
      A(II) = H1
      IF (I .EQ. N)   GOTO 80
      BL = I  .LE.  N - IBD + 1
      IP = I + 1
      JE = MIN0(N,I+IBD-1)
      DO 50, J=IP,JE
         LJ = LV(J)
         IJ = I + LJ
         H2 = A(IJ)
         IF (J .EQ. JE  .AND.  BL)   GOTO 50
         JA   = MAX0(1,J-IBD+1)
         JANZ = I - JA
         KI   = LI + JA
         KJ   = LJ + JA
         KIE  = KI + JANZ - 1
         KJE  = KJ + JANZ - 1
C         H2   = H2 - QVSKPX(A(KI),A(KJ),JANZ,1,1)
         H2   = H2 - DOT_PRODUCT(A(KI:KIE),A(KJ:KJE))
   50 A(IJ) = H2 / H1
      I = I + 1
      GOTO 20
   60 I   = 1
   70 IER = I
   80 RETURN
      END

