C     Last change:  BG   29 Jan 2002    3:01 pm
      SUBROUTINE CHOSOL(A,N,IBD,LV,X,M,Y)
C
C ======================================================================
C
C                               C H O S O L
C                               ===========
C                               ===========
C
C     S T A N D            :  14.01.88
C
C     B E A R B E I T E R  :  DR. GEIER, SCHOUREK, RENKEN
C                             DFVLR BRAUNSCHWEIG, INST. WB-SM
C
C     Z W E C K : GLEICHUNGSAUFLOESUNG NACH VORANGEGANGENER CHOLESKY-
C     =========== ZERLEGUNG EINER BANDMATRIX MIT DER DIMENSION (N,N)
C                 FUER M RECHTE SEITEN XALT: A(TRANSP)*A*XNEU = XALT.
C                 DIE MATRIX XNEU WIRD BERECHNET UND UEBERSCHREIBT DIE
C                 MATRIX XALT.
C
C
C     P A R A M E T E R : (REIHENFOLGE NACH DER PARAMETERLISTE)
C     ===================
C
C     EINGABE :
C     A....... (REAL*8)  CHOLESKY-ZERLEGTE EINER BANDMATRIX,
C                        BANDSTRUKTURIERTE OBERE DREIECKSMATRIX DER
C                        DIMENSION (N,N), LINEAR GESPEICHERT,
C                        S. HINWEISE
C     N....... (INTEGER) ANZAHL DER ZEILEN DER MATRIX A
C     IBD..... (INTEGER) BANDBREITE DER MATRIX A, IBD < N + 1
C                        (ANZAHL DER NEBENDIAGONALEN EINES DREIECKS + 1
C                        (HAUPTDIAGONALE))
C     LV...... (INTEGER) VEKTOR DER LAENGE N, ZUR INDEXBERECHNUNG DER
C                        MATRIX A, S. HINWEISE
C     X....... (REAL*8)  MATRIX DER GROESSE (N,M), RECHTE SEITE DES
C                        GLEICHUNGSSYSTEMS
C     M....... (INTEGER) ANZAHL DER RECHTEN SEITEN
C
C     AUSGABE :
C     X....... (REAL*8)  ERGEBNISMATRIX DER GROESSE (N,M)
C
C     HILFSGROESSEN:
C     Y....... (REAL*8)  VEKTOR DER LAENGE N
C
C     U N T E R P R O G R A M M E :
C     =============================
C     QVSKPX.. SKALARPRODUKT, DOPPELTGENAU, INTERN VIERFACH GENAU,
C
C     H I N W E I S E :
C     =================
C
C     DIE OBERE DREIECKSMATRIX R IST LINEAR SPALTENWEISE GESPEICHERT,
C     UND ZWAR NUR DAS BAND.
C     INDEXBERECHNUNG: ELEMENT (I,J) IST A(LV(J)+I)
C     DIMENSION BEI IBD = N: N*(N+1)/2
C               BEI IBD < N: N*IBD - (IBD*(IBD-1)/2)
C
C ======================================================================
C
      DOUBLE PRECISION A(1), X(N,M), Y(N), YI, RANN
      DIMENSION        LV(N)
C
      NN   = LV(N) + N
      RANN = 1.D0 / A(NN)
      DO 50, J=1,M
         Y(1) = X(1,J) / A(1)
         DO 10, I=2,N
            LI   = LV(I)
            KA   = MAX0(1,I-IBD+1)
            KANZ = I - KA
            KAL  = KA + LI
            KEL  = KAL + KANZ - 1
            KAE  = KA + KANZ - 1
C            Y(I) = (X(I,J) - QVSKPX(A(KA+LI),Y(KA),KANZ,1,1)) / A(I+LI)
            Y(I) = (X(I,J) - DOT_PRODUCT(A(KAL:KEL),Y(KA:KAE)))/A(I+LI)
   10    CONTINUE
         Y(N) = Y(N) * RANN
         DO 30, I=N-1,1,-1
            YI = Y(I)
            KE = MIN0(N,I+IBD-1)
            DO 20, K=I+1,KE
               IK = LV(K) + I
               YI = YI - A(IK)*Y(K)
   20       CONTINUE
            II = LV(I)+I
            Y(I) = YI / A(II)
   30    CONTINUE
         DO 40, I=1,N
            X(I,J) = Y(I)
   40    CONTINUE
   50 CONTINUE
      RETURN
      END
