C     Last change:  BG   21 Feb 2005    2:38 pm
      SUBROUTINE IUORIV( I, J, IFUN, IU, IER )
C
C ######################################################################
C
C                               I U O R I V
C                               ###########
C                               ###########        COPYRIGHT BY DLR 1994
C
C     S T A T E    :    11.12.2001
C
C     A U T H O R  :    DR.-ING. BODO GEIER
C                       DLR BRAUNSCHWEIG, INST. WB-SM
C
C     P U R P O S E :   FORM MATRIX CONTAINING NUMBERS OF DEGREES OF
C     ===============   FREEDOM FOR AN ELEMENT (SUBDIVISION) OF BEOS
C
C     A R G U M E N T S :  ALL ARGUMENTS ARE TYPE INTEGER
C     ===================
C
C     INPUT   :
C     I, J              INDEX COMBINATION OF ELEMENT
C                       FIRST INDEX INCREASES IN Y-DIRECTION,
C                       SECOND INDEX INCREASES IN X-DIRECTION
C     IFUN              FUNCTION KEY (1 FOR W, 2 FOR U ...)
C
C     OUTPUT  :
C     IU(4,4)           RESULTING MATRIX. LET X POINT TOWARDS RIGHT,
C                       Y POINT DOWNWARD. THEN
C                       ENTRIES (1,1), (1,2), (2,1), (2,2) REFER TO
C                       UPPER LEFT CORNER;
C                       ENTRIES (3,1), (3,2), (4,1), (4,2) REFER TO
C                       LOWER LEFT CORNER;
C                       ENTRIES (1,3), (1,4), (2,3), (2,4) REFER TO
C                       UPPER RIGHT CORNER;
C                       ENTRIES (3,3), (3,4), (4,3), (4,4) REFER TO
C                       LOWER RIGHT CORNER.
C                       ENTRIES (1,1), (1,3), (3,1), (3,3) POINT TO
C                       FUNCTION VALUES;
C                       ENTRIES (1,2), (1,4), (3,2), (3,4) POINT TO
C                       X-DERIVATIVES;
C                       ENTRIES (2,1), (2,3), (4,1), (4,3) POINT TO
C                       Y-DERIVATIVES;
C                       ENTRIES (2,2), (2,4), (4,2), (4,4) POINT TO
C                       MIXED SECOND DERIVATIVES.
C     IER               ERROR INDEX, SET .NE. 0 IN CASE OF
C                       UNCONSISTENT ARGUMENTS
C
C     C O M M O N B L O C K S :
C     =========================
C
C     I N P U T   :
C     COMMON /MAN/    IX,IY,IMAX,ITER,NEIG,ITAB,NEIGA,MEMPRI,X2PRI,
C    *                INFEIG,SYMEIG,TEXT(20)
C     USED:
C     IX                NUMBER OF DEGREES OF FREEDOM FOR NORMAL
C      COMMON /F07INA/ KU(5),MX,MY,A(30),B(30),LEIT(31,31),
C     *                IBDX,IBDY,KNFUN
C     USED:
C     KU(5)             FUNCTION TYPES INVOLVED
C     MX, MY            NUMBER OF NODES IN X- AND Y-DIRECTION
C     LEIT(31,31)       NUMBERING OF NODES I,J
C     KNFUN             NUMBER OF FUNCTION TYPES INVOLVED
C     COMMON /F07MAT/LM(5,4,130)
C     LM(5,4,130)       KEY MATRIX
C
C     O U T P U T :
C     COMMON/DATRAN/NREAD,NWRITE,NSTORE
C     USED:
C     NWRITE            NUMBER OF OUTPUT CHANNEL
C
C ######################################################################
C
#ifndef MBEOS5
     INCLUDE '#PRAMS'
#endif
C
C         / M A N /
C         =========
      INTEGER :: IX,IY,ITER,NEIGIT,ITAB,NEIGRQ
      LOGICAL MEMPRI,X2PRI
      COMMON /MAN/ IX,IY,ITER,NEIGIT,ITAB,NEIGRQ,MEMPRI,X2PRI
C
      INTEGER, DIMENSION (1:5)::KU
      INTEGER, DIMENSION(1:P_RTNDS,1:P_RTNDS):: LEIT
      REAL, DIMENSION(1:P_RTNDS-1):: A, B
      COMMON /F07INA/ KU,MX,MY,A,B,LEIT,IBDX,IBDY,KNFUN
C
      INTEGER, DIMENSION(1:5,1:4,1:P_NODES):: LM
      COMMON /F07MAT/ LM
C
      INTEGER:: NREAD,NWRITE,NSTORE
      COMMON /DATRAN/ NREAD,NWRITE,NSTORE
C
      LOGICAL         TEST,KU4
      DIMENSION       IU(4,4)
C
      IER   = 0
C
C     TEST FOR VALID NODE
      IF((I.LT.1.OR.I.GT.(MY-1)) .OR. (J.LT.1.OR.J.GT.(MX-1))) THEN
         WRITE (NWRITE,7000)
         IER = 1
         GOTO 999
      END IF
C
C     TEST FOR VALID FUNCTION
      KU4 = KNFUN .EQ. 3 .AND. KU(2) .EQ. 4

      TEST = (IFUN .GE. 1.AND. IFUN .LE.5)
     *        .AND. (((IFUN .EQ.1 .OR. KNFUN .EQ. 5)
     *       .OR. (KU4 .AND. IFUN .GE. 4))
     *       .OR. (KNFUN .EQ. 3 .AND. IFUN .LE. 3))
C
      IF (TEST) THEN
C           UPPER LEFT NODE
            NODE = LEIT(I,J)
            IU(1,1) = LM(IFUN,1,NODE)
            IU(1,2) = LM(IFUN,2,NODE)
            IU(2,1) = LM(IFUN,3,NODE)
            IU(2,2) = LM(IFUN,4,NODE)
C
C           UPPER RIGHT NODE
            NODE = LEIT(I,J+1)
            IU(1,3) = LM(IFUN,1,NODE)
            IU(1,4) = LM(IFUN,2,NODE)
            IU(2,3) = LM(IFUN,3,NODE)
            IU(2,4) = LM(IFUN,4,NODE)
C
C           LOWER LEFT NODE
            NODE = LEIT(I+1,J)
            IU(3,1) = LM(IFUN,1,NODE)
            IU(3,2) = LM(IFUN,2,NODE)
            IU(4,1) = LM(IFUN,3,NODE)
            IU(4,2) = LM(IFUN,4,NODE)
C
C           LOWER RIGHT NODE
            NODE = LEIT(I+1,J+1)
            IU(3,3) = LM(IFUN,1,NODE)
            IU(3,4) = LM(IFUN,2,NODE)
            IU(4,3) = LM(IFUN,3,NODE)
            IU(4,4) = LM(IFUN,4,NODE)
C           IF (IFUN .NE. 1) THEN
C              DO K=1,4
C              DO L=1,4
C                 IF (IU(K,L) .NE. 0) IU(K,L) =IU(K,L) - IX
C              END DO
C              END DO
C           END IF
      ELSE
            DO K=1,4
            DO L=1,4
               IU(K,L) = 0
            END DO
            END DO
            IER = 2
      END IF
C
  999 RETURN
 7000 FORMAT(/' MESSAGE FROM SUBROUTINE IUORIV: INCONSISTENT ARGUMENT',
     *       /' WRONG FUNCTION INDEX, OR SUBDIVISION OUT OF RANGE')
C
C     END OF SUBROUTINE IUORIV
      END

