      SUBROUTINE JACOB2(A,N,LVTRI,EPS,YS,IER)
C
C ======================================================================
C
C                               J A C O B 2
C                               ===========
C                               ===========
C
C     S T A N D            :  09.01.92
C
C     B E A R B E I T E R  :  DR. GEIER, SCHOUREK, RENKEN
C                             DFVLR BRAUNSCHWEIG, INST. WB-SM
C
C     Z W E C K : BERECHNUNG DER EIGENWERTE UND EIGENVEKTOREN DER
C     =========== SYMMETRISCHEN MATRIX A MIT DER JACOBI-METHODE.
C                 DURCH EINE FOLGE VON TRANSFORMATIONEN WIRD DIE
C                 MODALMATRIX YS BERECHNET, DIE A AUF DIAGONALFORM
C                 TRANSFORMIERT:  YS(TRANSP)*A*YS = D
C
C
C     P A R A M E T E R : (REIHENFOLGE NACH DER PARAMETERLISTE)
C     ===================
C
C     EINGABE :          "REAL*8" STATT "DOUBLE PRECISION" GESCHRIEBEN
C     A....... (REAL*8)  (N,N)-MATRIX, DEREN EIGENWERTE UND EIGEN-
C                        VEKTOREN GESUCHT WERDEN, S. HINWEISE
C     N....... (INTEGER) DIMENSION DER QUADRATISCHEN MATRIZEN A UND YS
C                        S.A. HINWEISE
C     LVTRI... (INTEGER) VEKTOR DER LAENGE N, ZUR INDEXBERECHNUNG DER
C                        MATRIX A, S.A. HINWEISE
C     EPS..... (REAL*8)  GENAUIGKEITSSCHRANKE.
C                        WENN ALLE AUSSERDIAGONALELEMENTE DER MATRIX A
C                        KLEINER ALS
C                        EPS * (SUMME DER DIAGONALELEMENTE VON A) / N,
C                        DANN IST DIE MATRIX MIT GENUEGENDER GENAUIGKEIT
C                        AUF DIAGONALFORM GEBRACHT.
C
C     AUSGABE :
C     A....... (REAL*8)  (N,N)-DIAGONALMATRIX D. AUF DER DIAGONALEN
C                        STEHEN DIE EIGENWERTE VON A, S.A. HINWEISE
C     YS...... (REAL*8)  (N,N)-MODALMATRIX.
C                        DIE SPALTEN VON YS ENTHALTEN JEWEILS
C                        EINEN EIGENVEKTOR VON A.
C     IER..... (INTEGER) FEHLERPARAMETER
C                        IER = 0: FEHLERFREI GELAUFEN
C                        IER =-1: KEINE KONVERGENZ NACH 2*N**2 TRANSFOR-
C                                 MATIONEN. ABBRUCH DES VERFAHRENS
C                        IER =-2: MATRIX A IST NULLMATRIX
C
C     H I N W E I S E :
C     =================
C
C     DIE MATRIX A IST LINEAR SPALTENWEISE GESPEICHERT, UND ZWAR NUR
C     DAS OBERE DREIECK. DAS UP GILT IN DIESER FORM FUER VOLL BESETZTE
C     MATRIZEN (KEINE BANDSTRUKTUR)
C     INDEXBERECHNUNG: ELEMENT (I,J) IST A(LVTRI(J)+I)
C     DIMENSION      : N*(N+1)/2
C
C ======================================================================
C
      DOUBLE PRECISION EPS,A(*),YS(N,N),SP,RMAX,RACT,AIJ,DIAGI,DIAGJ,TH,
     *                 T,TQ,CC,C,SS,S,CS,ZCSA,AII,AJJ,YMI,YMJ,DABS,
     *                 DSIGN,DSQRT,SNORM,SH
      DIMENSION        LVTRI(N)
      IER = 0
C
C     INITIALISE YS AND CALCULATE A MATRIX NORM (MAX LINE NORM)
C
      SNORM = 0.0D0
      DO 20  J=1,N
         YS(J,J) = 1.D0
         LVJ = LVTRI(J)
         SH = ABS(LVJ+J)
         IF (J .GT. 1) THEN
            DO 10  I=1,J-1
               YS(I,J) = 0.D0
               SH = SH + ABS(A(LVJ+I))
   10       CONTINUE
         END IF
         IF (J .LT. N) THEN
            DO 15, I=J+1,N
               YS(I,J) = 0.D0
               SH = SH + ABS(A(LVTRI(I)+J))
   15       CONTINUE
         END IF
         SNORM = MAX(SNORM,SH)
   20 CONTINUE
      IF (SNORM .LT. 1.D-10*EPS) THEN
C        ALL ELEMENTS OF MATRIX A ARE ZERO
         IER = -2
         GOTO 160
      END IF
      SP = EPS*SNORM/N
C
      IF (N .EQ. 1) GOTO 160
      NBR    = 0
      MAXNBR = N*N*2
C
C*******   FIND GREATEST OFF - DIAGONAL ELEMENT            ***********
C
      IPIV = 1
      JPIV = 2
      KPIV = 2
   30 CONTINUE
      RMAX = 0.0D0
      DO 50  J=2,N
         LVJ = LVTRI(J)
         JM1 = J - 1
         DO 40  I=1,JM1
            K    = LVJ + I
            RACT = DABS(A(K))
            IF (RACT .LE. RMAX) GOTO 40
            KPIV = K
            RMAX = RACT
            IPIV = I
            JPIV = J
   40    CONTINUE
   50 CONTINUE
C
C ******** TEST FOR TERMINATION **************************************
C
      IF (RMAX .LT. SP) GOTO 160
      IF (NBR .GT. MAXNBR) GOTO 150
C
C ********  BEGIN OF ROTATION  ***************************************
C
      LVI   = LVTRI(IPIV)
      LI    = LVI + IPIV
      LVJ   = LVTRI(JPIV)
      LJ    = LVJ + JPIV
      AIJ   = A(KPIV)
      DIAGI = A(LI)
      DIAGJ = A(LJ)
      IM1   = IPIV - 1
      IP1   = IPIV + 1
      TH    = (DIAGJ - DIAGI)*0.5D0/AIJ
      IF (DABS(TH).LT.1.0D-30) GOTO 60
      T     = 1.0D0/(TH + DSIGN(DSQRT(1.0D0 + TH**2),TH))
      TQ    = T*T
      CC    = 1.0D0/(1.D0 + TQ)
      C     = DSQRT(CC)
      SS    = TQ*CC
      S     = T*C
      CS    = T*CC
      GOTO 70
   60 CONTINUE
      CC    = 0.5D0
      SS    = 0.5D0
      CS    = 0.5D0
      C     = DSQRT(CC)
      S     = C
C
C     **  TRANSFORMATION   ***********************************
C
   70 CONTINUE
      ZCSA    = 2.0D0*CS*AIJ
      A(KPIV) = 0.0D0
      A(LI)   = CC*DIAGI - ZCSA + SS*DIAGJ
      A(LJ)   = SS*DIAGI + ZCSA + CC*DIAGJ
C
      IF (IM1 .LT. 1) GOTO 90
      DO 80  M=1,IM1
         II    = LVI + M
         JJ    = LVJ + M
         AII   = A(II)
         AJJ   = A(JJ)
         A(II) = C*AII - S*AJJ
         A(JJ) = S*AII + C*AJJ
   80 CONTINUE
   90 JM1 = JPIV - 1
C
      IF (JM1 .LT. IP1) GOTO 110
      DO 100  M=IP1,JM1
         II    = LVJ + M
         JJ    = LVTRI(M) + IPIV
         AII   = A(II)
         AJJ   = A(JJ)
         A(II) = C*AII + S*AJJ
         A(JJ) = C*AJJ - S*AII
  100 CONTINUE
  110 JP1 = JPIV + 1
C
      IF (JPIV .EQ. N) GOTO 130
      DO 120  M=JP1,N
         II    = LVTRI(M)
         JJ    = II + JPIV
         II    = II + IPIV
         AII   = A(II)
         AJJ   = A(JJ)
         A(II) = C*AII - S*AJJ
         A(JJ) = S*AII + C*AJJ
  120 CONTINUE
C
C     *******  TRANSFORMATION OF MODAL MATRIX  **************
C
  130 DO 140  M=1,N
         YMI        = YS(M,IPIV)
         YMJ        = YS(M,JPIV)
         YS(M,IPIV) = C*YMI - S*YMJ
         YS(M,JPIV) = S*YMI + C*YMJ
  140 CONTINUE
      NBR = NBR + 1
      GOTO 30
  150 IER = -1
  160 RETURN
C     END OF SUBROUTINE JACOB2
      END
