C     Last change:  BG   28 Mar 2007   12:17 pm
      SUBROUTINE KUORKV(XORY,II,JJ,IFUN,KV,  IER)
C
C ######################################################################
C
C                               K U O R K V
C                               ###########
C                               ###########        COPYRIGHT BY DLR 2000
C
C     S T A T E    :  11.12.2001
C
C     A U T H O R  :  DR. B.GEIER, DLR BRAUNSCHWEIG, INST. WB-SM
C
C     P U R P O S E : ESTABLISH MATRIX CONTAINING NUMBERS OF DEGREES OF
C     =============== FREEDOM FOR A STIFFENER ELEMENT OF BEOS
C
C     A R G U M E N T S :
C     ===================
C
C     INPUT   :
C     XORY           CHARACTER*1
C                    MUST BE 'X' OR 'Y' FOR STIFFENER
C                    IN X- OR Y-DIRECTION, RESPECTIVELY
C     II, JJ         INDEX COMBINATION OF ELEMENT
C                    FIRST INDEX INCREASES TRANSVERSE TO STIFFENER,
C                    SECOND INDEX INCREASES IN STIFFENER DIRECTION.
C     IFUN           FUNCTION KEY (1 FOR W, 2 FOR U ...)
C
C     OUTPUT  :
C     KV(2,4)        RESULING MATRIX.
C
C                    ENTRIES (1,1), (1,2), (2,1), (2,2) REFER TO
C                    NODE WITH LOWER COORDINATE VALUE (FIRST NODE)
C                    ENTRIES (1,3), (1,4), (2,3), (2,4) REFER TO
C                    NODE WITH HIGHER COORDINATE VALUE (SECOND NODE)
C
C                    ENTRIES (1,1), (1,3) POINT TO FUNCTION VALUES;
C                    ENTRIES (1,2), (1,4) POINT TO DERIVATIVES IN
C                    THE DIRECTION OF THE STIFFENER
C                    ENTRIES (2,1), (2,3) POINT TO DERIVATIVES
C                    TRANSVERSE TO STIFFENER DIRECTION
C                    ENTRIES (2,2), (2,4) POINT TO
C                    MIXED SECOND DERIVATIVES.
C
C     IER            ERROR INDEX
C                       IER .EQ. 0:  NO ERROR
C                       IER .EQ. 2:  ARGUMENTS I,J SPECIFY WRONG NODE
C                       IER .EQ. 1:  ARGUMENT XORY NEITHER 'X' NOR 'Y'
C
C     C O M M O N B L O C K S :
C     =========================
C
C     I N P U T   :
C     COMMON /INA/ KU(5),MX, MY, A(30), B(30), LEIT(31,31),
C                                        IBDX, IBDY, KNFUN
C     USED:
C     MX, MY         NUMBER OF NODES IN X- AND Y-DIRECTION
C     LEIT(31,31)    NUMBERING OF NODES I,J
C
C     COMMON /F07MAT/LM(5,4,130)
C     LM(5,4,130)    KEY MATRIX
C
C     O U T P U T :
C     COMMON/DATRAN/ NREAD, NWRITE, NSTORE
C     USED:
C     NWRITE         NUMBER OF OUTPUT CHANNEL
C
C ######################################################################
C
C 		PARAMETERS USED IN PROGRAM BEOS:
#ifndef MBEOS5
     INCLUDE '#PRAMS'
#endif
C
C         / M A N /
C         =========
      INTEGER :: IX,IY,ITER,NEIGIT,ITAB,NEIGRQ
      LOGICAL MEMPRI,X2PRI
      COMMON /MAN/ IX,IY,ITER,NEIGIT,ITAB,NEIGRQ,MEMPRI,X2PRI
C
      INTEGER, DIMENSION (1:5)::KU
      INTEGER, DIMENSION(1:P_RTNDS,1:P_RTNDS):: LEIT
      REAL, DIMENSION(1:P_RTNDS-1):: A, B
      COMMON /F07INA/ KU,MX,MY,A,B,LEIT,IBDX,IBDY,KNFUN
C
      INTEGER, DIMENSION(1:5,1:4,1:P_NODES):: LM
      COMMON /F07MAT/ LM
C
      INTEGER:: NREAD,NWRITE,NSTORE
      COMMON /DATRAN/ NREAD, NWRITE, NSTORE
C
      CHARACTER*1 XORY
      LOGICAL CV
      INTEGER, DIMENSION (2,4):: KV
C
C######################################################################
C      PRINT*, 'in KUORKV: XORY, ii,jj, MX,MY'
C      PRINT*, XORY, II, JJ, MX, MY
C#####################################################################
      IER   = 0
C     CHECK NODE
      IF (XORY .EQ. 'X') THEN
         CV = (II.LT.1.OR.II.GT.MY) .OR. (JJ.LT.1.OR.JJ.GT.MX)
      ELSEIF (XORY .EQ. 'Y') THEN
         CV = (JJ.LT.1.OR.JJ.GT.MY) .OR. (II.LT.1.OR.II.GT.MX)
      ELSE
         IER = 1
         WRITE ( NWRITE,7000)
         WRITE ( NWRITE,7002)
         GOTO 999
      END IF
      IF (CV) THEN
         WRITE ( NWRITE,7000)
         WRITE ( NWRITE,7001) II,JJ, XORY
         IER = 2
         GOTO 999
      END IF
C
C     STIFFENER IN X-DIRECTION
      IF (XORY .EQ. 'X') THEN
         I = II
         J = JJ
C        FIRST NODE
         NODE = LEIT(I,J)
         KV(1,1) = LM(IFUN,1,NODE)
         KV(1,2) = LM(IFUN,2,NODE)
         KV(2,1) = LM(IFUN,3,NODE)
         KV(2,2) = LM(IFUN,4,NODE)
C
C        SECOND NODE
         NODE = LEIT(I,J+1)
         KV(1,3) = LM(IFUN,1,NODE)
         KV(1,4) = LM(IFUN,2,NODE)
         KV(2,3) = LM(IFUN,3,NODE)
         KV(2,4) = LM(IFUN,4,NODE)
C
C     STIFFENER IN Y-DIRECTION
      ELSE
         I = JJ
         J = II
C        FIRST NODE
         NODE = LEIT(I,J)
         KV(1,1) = LM(IFUN,1,NODE)
         KV(1,2) = LM(IFUN,3,NODE)
         KV(2,1) = LM(IFUN,2,NODE)
         KV(2,2) = LM(IFUN,4,NODE)
C
C        SECOND NODE
         NODE = LEIT(I+1,J)
         KV(1,3) = LM(IFUN,1,NODE)
         KV(1,4) = LM(IFUN,3,NODE)
         KV(2,3) = LM(IFUN,2,NODE)
         KV(2,4) = LM(IFUN,4,NODE)
      END IF
C      IF (IFUN .ne. 1) THEN
C         DO, I=1,2
C            DO, J=1,4
C               IF (KV(I,J) .NE. 0) THEN
C                  KV(I,J) = KV(I,J) - IX
C               END IF
C            END DO
C         END DO
C      END IF
  999 RETURN
 7000 FORMAT(/' MESSAGE FROM SUBROUTINE KUORKV:')
 7001 FORMAT(' SECOND AND THIRD ARGUMENT SPECIFY WRONG NODE',2I4,
     *       'STIFFENER DIRECTION IS ',A1)
 7002 FORMAT(' FIRST ARGUMENT NEITHER ''X'' NOR ''Y'' ')
C
C     END OF SUBROUTINE KUORKV
      END

