C     Last change:  BG   21 Jun 2004   10:30 am
      SUBROUTINE PROBTM(A,N,IBD,LV,B,M1,M,C)
C ======================================================================
C
C                               P R O B T M
C                               ===========
C                               ===========
C
C     S T A T E            :  16.01.2002
C
C     A U T H O R S        :  DR. GEIER, K. SCHOUREK, U. RENKEN
C                             DLR BRAUNSCHWEIG, INST. WB-SM
C
C     P U R P O S E : MATRIX PRODUCT  A*B = C
C     =============   A IS SYMMETRIC BAND MATRIX, STORED AS VECTOR.
C                     B AND C ARE (N,M)-RECTANGULAR MATRICES.
C
C                     DOT_PRODUCT is used (rather than QVSKPX) to
C                     compute inner product
C
C     P A R A M E T E R :
C     ===================
C
C     INPUT :
C     A....... (REAL*8)  SYMMETRIC (N,N)-MATRIX, STORED AS VECTOR
C                        (CF COMMENTS)
C     N....... (INTEGER) NUMBER OF ROWS OF MATRICES A, B AND C
C     IBD..... (INTEGER) BANDWIDTH OF MATRIX A, IBD < N + 1
C                        (MAX NUMBER OF OFF-DIAGONAL ELEMENTS IN A ROW OF
C                         UPPER TRIANGLE + 1 )
C     LV...... (INTEGER) VECTOR OF LENGTH N, NEEDED TO FIND ELEMENTS OF
C                        MATRIX A, CF COMMENTS.
C     B....... (REAL*8)  (N,M)-MATRIX
C     M1       (INTEGER) Number of first column to be computed in C
C     M....... (INTEGER) NUMBER OF CCOLUMNS OF MATRICES B AND C
C
C     OUTPUT :
C     C....... (REAL*8)  (N,M)-MATRIX, RESULT OF PRODUCT A*B
C
C
C     C O M M E N T S :
C     =================
C     THE ELEMENTS IN THE UPPER TRIANGLE OF MATRIX A, INCLUDING THE
C     DIAGONAL, ARE STORED IN COLUMN-WISE ORDER
C     (ELEMENTS WITHIN THE BAND ONLY).
C     DETERMINATION OF INDEX: IF (I.LE.J)THEN
C                                ELEMENT (I,J) IS A(LV(J)+I)
C                             ELSE
C                                ELEMENT (I,J) IS A(LV(I)+J)
C                             END IF
C
C ======================================================================
C
      DOUBLE PRECISION,INTENT(IN), DIMENSION(*)::       A
      DOUBLE PRECISION,INTENT(IN), DIMENSION(1:N,1:M):: B
      DOUBLE PRECISION,INTENT(OUT), DIMENSION(1:N,1:M)::C
      INTEGER,INTENT(IN), DIMENSION(1:N)::              LV
      INTEGER, INTENT(IN)::                             N, IBD, M1, M
C
      INTEGER:: I, J, K, L, IBD1, KL, KE
      INTEGER, DIMENSION(1:N):: IXA
C
      IBD1 = IBD - 1
C
      DO, I=1,N
         IF (I .LE. IBD) THEN
            KL = 1;  KE = MIN(N,I+IBD1)
         ELSE
         	KL = I - IBD1;   KE = MIN(N,I+IBD1)
         END IF
         DO,  K=KL,KE
            IF (K .LE. I) THEN
               IXA(K) = LV(I) + K
            ELSE
               IXA(K) = LV(K) + I
            END IF
         END DO
         DO,   K=M1,M
            C(I,K) = DOT_PRODUCT
     *       (A((/(IXA(L),L=KL,KE)/)),B((/(J,J=KL,KE)/),K))
         END DO
      END DO
      CONTINUE
      RETURN
C     END SUBROUTINE PROPBTM
      END
