C     Last change:  BG    8 Sep 2005    3:50 pm
      SUBROUTINE SIMVIT(NEIGRQ,NEIGIT,A,B,N,IBDA,IBDB,LVBNDA,LVBNDB,EPS,
     *                  ELAM,X,JAC,IP,LVTRI ,IER)
C
C ======================================================================
C
C                               S I M V I T
C                               ===========
C                               ===========
C
C     S T A T E            : 05.07.2004
C
C     A U T H O R S        :  DR. GEIER, SCHOUREK, RENKEN
C                             DFVLR BRAUNSCHWEIG, INST. WB-SM
C
C     P U R P O S E : CALCULATION OF EIGENVALUES AND EIGENVECTORS
C     ===========     FOR THE GENERAL EIGENVALUE PROBLEM
C                        (A - LAMBDA * B) * X = 0
C                     BY INVERSE SUBSPACE ITERATION. THE SUBROUTINE
C                     DETERMINES SOME SOLUTIONS AT THE LOWER END OF
C                     THE SPECTRUM.
C
C     A R G U M E N T S : (IN THE ORDER OF THE PARAMETER LIST)
C     ===================
C
C     INPUT:
C     NEIGRQ.. (INTEGER) NUMBER OF REQUIRED EIGENVALUES AND
C                        EIGENVECTORS, NEIGRQ < N - 2
C     NEIGIT.. (INTEGER) DIMENSION OF SUBSPACE,
C                        N > NEIGIT > NEIGRQ + 1
C     A....... (REAL*8)  | PAIR OF MATRICES DEFINING THE PROBLEM.
C     B....... (REAL*8)  | FOR STORAGE MODE SEE INDICATIONS!
C
C     N....... (INTEGER) DIMENSION OF THE PROBLEM
C     IBDA.... (INTEGER) BANDWIDTH OF MATRIX A, IBDA < N + 1
C                        (MAX. NUMBER OF OFF-DIAGONAL ELEMENTS IN UPPER
C                        OR LOWER TRIANGLE)
C     IBDB.... (INTEGER) BANDWIDTH OF MATRIX B, IBDB < N + 1
C
C     LVBNDA.. (INTEGER) VECTOR OF LENGTH N, USED FOR THE DETERMINATION
C                        OF INDECES FOR MATRIX A, SEE INDICATIONS!
C     LVBNDB.. (INTEGER) VECTOR OF LENGTH N, USED FOR THE DETERMINATION
C                        OF INDICES FOR MATRIX B, SEE INDICATIONS!
C     EPS..... (REAL*8)  PRECISION LIMIT (INITIAL VALUE),
C                        WILL BE UPDATED AT EACH ITERATION STEP
C     LVTRI... (INTEGER) VECTOR OF LENGTH NEIGIT, USED TO DETERMINE
C                        INDICES OF THE INTERNAL MATRIX BS (FULL TRIANG).
C     OUTPUT:
C     EPS..... (REAL*8)  PRECISION LIMIT  AT WHICH THE ITERATION STOPPED
C     ELAM.... (REAL*8)  VECTOR OF  LENGTH NEIGIT CONTAINING THE
C                        CALCULATED EIGENVALUES LAMBDA
C     X....... (REAL*8)  (N,NEIGIT)-MATRIX THE COLUMNS OF WHICH ARE THE
C                        CALCULATED EIGENVECTORS
C     JAC..... (INTEGER) NUMBER OF CALLS TO SUBROUTINE JACOB2
C     IP...... (INTEGER) NUMBER OF INVERSE POWER ITERATIONS
C     IER..... (INTEGER) ERROR INDICATOR
C                        IER = 0: NO ERROR
C                        IER =-1: WARNING! NO CONVERGENCE IN JACOB2,
C                                 JACOBI ITERATIONS STOPPED. CALCULATION
C                                 PROCEEDS WITH STATE REACHED SO FAR.
C                        IER =-2: ARGUMENT TRANFERRED TO JACOB2 IS ZERO
C                                 MATRIX
C                        IER =-101: MATRIX A INDEFINIT. BECAME APPARENT
C                                   AT ORTHO-NORMALISATION
C                        IER > 0: CHOLESKY-DECOMPOSITION NOT POSSIBLE.
C                                 MATRIX IS NOT POSITIVE DEFINITE.
C                                 ELEMENT LEADING TO STOP:
C                                 A(IER)
C
C
C     S U B R O U T I N E S     C A L L L E D :
C     =============================
C     CHODEC.. CHOLESKY-DECOMPOSITION OF A MATRIX
C     CHOSOL.. EQUATION SOLUTION WITH CHOLESKY-DECOMPOSED MATRIX
C     JACOB2.. SOLUTION OF A GENERAL EIGENVALUE PROBLEM BY JACOBI METH.
C     PROBTM.. MATRIX PRODUCT WITHE LINEARLY STORED SYMM. MATRIX
C
C     I N D I C A T I O N S :
C     =======================
C
C     THE MATRICES A AND B ARE STORED LINEARLY COLUMN-WISE.
C     ONLY THE BAND IN THE UPPER TRIANGLE IS RETAINED.
C     INDEX: ELEMENT (I,J) IS A(LVBNDA(J)+I)
C                          OR B(LVBNDB(J)+I), RESPECTIVELY
C     DIMENSION OF MATRIX A:
C               FOR IBDA = N: N*(N+1)/2
C               FOR IBDA < N: N*IBDA - (IBDA*(IBDA-1)/2)
C     DIMENSION OF MATRIX B CORRESPONDING.
C
C     AFTER EACH INVERSE POWER ITERATION STEP THE PRECISION
C     REQUIREMENT IS RELAXED, I.E. EPS IS INCREASED BY A CERTAIN
C     PERCENTAGE. IN THAT WAY THE PROCEDURE WILL ALWAYS COME TO AN END
C     WITHOUT PUTTING A LIMIT ON THE TOTAL NUMBER OF ITERATIONS.
C     TO KEEP CONTROL ON WHETHER A USEFUL SOLUTION HAS BEEN OBTAINED
C     CHECK THE OUTPUT VALUE OF EPS!
C
C ======================================================================
C
      IMPLICIT NONE
C     TYPES AND DIMENSIONS OF DUMMY ARGUMENTS
      INTEGER:: NEIGRQ, NEIGIT, N, IBDA, IBDB, JAC, IP, IER
      INTEGER, DIMENSION(*):: LVBNDA, LVBNDB, LVTRI
      DOUBLE PRECISION, DIMENSION(*):: A, B, ELAM
      DOUBLE PRECISION, DIMENSION(1:N,1:NEIGIT):: X
C
C     AUXILIARY ARRAYS
      DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:):: Y, H
      DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:):: CH, BS
      INTEGER, ALLOCATABLE, DIMENSION(:):: LH
C
      DOUBLE PRECISION,DIMENSION(1:NEIGIT,1:NEIGIT):: YS
      INTEGER,DIMENSION(1:NEIGIT)::  LH1
      DOUBLE PRECISION,DIMENSION(N):: RNDNMB
      DOUBLE PRECISION:: EPS, C, EGWR, PA, PB, EXACT, DWN, UPP, QUOMAX
      INTEGER:: I,J,K,L,M,II,IJ,JJ,MM,KK,NQ,NS,IIA,IIB,KEM,KM1,KP1,KRV,
     *          NSM1,NITM1,KANF
      INTEGER seedSize
      INTEGER, ALLOCATABLE :: seedArray (:)
      INTEGER:: NREAD, NWRITE, NSTORE
      COMMON /DATRAN/ NREAD,NWRITE,NSTORE
C
C***********************************************************************
C     SET INITIAL VALUES
      IER = 0
C      CALL RANDOM_SEED ( )  ! Processor reinitializes the seed
                            ! randomly from the date and time
      CALL RANDOM_SEED (SIZE = seedSize)  ! I is set to the size of
                                   ! the seed array
      ALLOCATE (seedArray(seedSize))
      seedArray = 5
C      WRITE(*,*) seedArray
      CALL RANDOM_SEED (PUT=seedArray(1:seedSize)) ! Sets seed from array
      DEALLOCATE (seedArray)

      EXACT  = 1.0D-8
C     EXACT  =  ACCURACY OF JACOBI'S METHOD
C
C     IP COUNTS POWER OPERATIONS
      IP = 0
C
C     JAC COUNTS EIGENVALUE SOLUTIONS IN SUBSPACE (CALL JACOB2)
      JAC = 0
C
      NITM1 = NEIGIT - 1
C
C     M SPECIFIES NO. OF POWER ITERATIONS BETWEEN CALLS TO JACOB2
      M = 3
C
C     KEM IS NO. OF CONVERGED EIGENVECTORS + 1
      KEM = 1
C
C     FORMING OF VECTOR LH
C     IT WILL CONTAIN THE NEIGIT LARGEST VALUES DABS(B(I,I)/A(I,I)).
      ALLOCATE(LH(N),CH(N*(N+1)/2),H(N,NEIGIT),Y(N,NEIGIT))
      ALLOCATE(BS(NEIGIT*(NEIGIT+1)/2))
C
      DO, I=1,N
         IIA = LVBNDA(I) + I
         IIB = LVBNDB(I) + I
         UPP = ABS(B(IIB))
         DWN = ABS(A(IIA))
         IF (DWN .LT. 1.D-20 .OR. DWN .LT. 1.D-20*UPP) THEN
            CH(I) = 1.D20
         ELSE
            CH(I) = UPP/DWN
         END IF
         LH(I) = I
      END DO
      DO, K=1,NITM1
         QUOMAX = CH(K)
         J      = K
         KP1    = K + 1
         DO, I=KP1,N
            IF (CH(I) .LT. QUOMAX) CYCLE
               QUOMAX = CH(I)
               J      = I
         END DO
         IF (J .NE. K) THEN
            KK    = LH(K)
            LH(K) = LH(J)
            LH(J) = KK
            CH(J) = CH(K)
            CH(K) = QUOMAX
         END IF
      END DO
C
C     MAKE START MATRIX X FOR MODE SHAPE ITERATION
C     FILL COLUMNS 1 TO NEIGIT-1 OF MATRIX X WITH RANDOM NUMBERS
      DO,  K=1,NITM1
         CALL RANDOM_NUMBER(RNDNMB)
         DO,  I=1,N
C            RNDNMB(I) = 1
            X(I,K) = RNDNMB(I)
         END DO
C        EMPHASIZE ONE ENTRY OF EACH COLUMN USING INFORMATION CONTAINED
C        IN LH
         I      = LH(K)
         X(I,K) = 2.D0
      END DO
C
C     CHOLESKY-DECOMPOSITION OF MATRIX A AFTER COPYING IT TO CH:
C     A:= R(TRANSP.)*R,      CH:= R, A UNCHANGED
C
      NQ = LVBNDA(N) + N
      DO,   I=1,NQ
         CH(I) = A(I)
      END DO
      CALL CHODEC(CH,N,IBDA,LVBNDA,IER)
         IF (IER .NE. 0) THEN
           WRITE (*,1058) IER
           WRITE (NWRITE,1058) IER
           IER = 1
           GOTO 999
         END IF
C
C     MATRIX PRODUCT Y = B*X
      CALL PROBTM(B,N,IBDB,LVBNDB,X(:,1:NITM1),1,NITM1,Y(:,1:NITM1))
C
C***********************************************************************
C     START OF PRINCIPLE LOOP
C
  90  CONTINUE
      NSM1 = NEIGIT - KEM !Number of columns which did not converge yet
C
C     ************* M SIMPLE INVERSE POWER ITERATIONS ******************
C     *****  Y=B*X !(except in first iteration); X=Y; X=A**(-1)*X  *****
C
         X(:,KEM:NITM1) = Y(:,KEM:NITM1)
         CALL CHOSOL(CH,N,IBDA,LVBNDA,X(:,KEM:NITM1),NSM1,X(:,NEIGIT))
C        COLUMN X(:,NEIGIT) USED AS SCRATCH
         EPS = EPS * 1.03D0
         IP  = IP + 1
         IF ( M .NE. 1 )  THEN
         DO,  MM=2,M
C           MATRIX PRODUCT Y = B*X
            CALL PROBTM(B,N,IBDB,LVBNDB,X(:,1:NITM1),
     *                          KEM,NITM1,Y(:,1:NITM1))
            X(:,KEM:NITM1) = Y(:,KEM:NITM1)
           CALL CHOSOL(CH,N,IBDA,LVBNDA,X(:,KEM:NITM1),NSM1,X(:,NEIGIT))
            EPS = EPS * 1.03D0
            IP  = IP + 1
         END DO
         END IF
C
C     STORE RANDOM NUMBERS IN COLUMN NEIGIT OF MATRIX X.
      CALL RANDOM_NUMBER(RNDNMB)
      X(:,NEIGIT) = RNDNMB
C
C     MATRIX PRODUCT Y = A*X, RESTRICTED TO COLUMN NEIGIT OF X
      CALL PROBTM(A,N,IBDA,LVBNDA,X,NEIGIT,NEIGIT,Y)
C     NOW FOR ALL COLUMNS FROM KEM TO NEIGIT Y = A*X
C
C     MAKE COLUMNS OF MATRIX X ORTHO-NORMAL WITH RESPECT TO MATRIX A.
      KANF = KEM
      IF (KEM .EQ. 1) THEN
C        NORMALIZE FIRST COLUMNS OF X AND Y
         C = DOT_PRODUCT(X(1:N,1),Y(1:N,1))
C         IF ( C .LT. 1.D-30) THEN
C            K = 1
C            WRITE (*,1059) K, IP
C            IER = -101
C            GOTO 999
C         END IF
         C = 1.D0/SQRT(C)
         DO,  I=1,N
            X(I,1) = C*X(I,1)
            Y(I,1) = C*Y(I,1)
         END DO
         KANF = 2
      END IF
C
C     ORTHO-NORMALIZE COLUMNS KANF TO NEIGIT OF MATRIX X
      DO,  K=KANF,NEIGIT
         KM1 = K - 1
C        1) Make column K orthogonal to preceding columns
         DO,  J=1,KM1
            C = DOT_PRODUCT(X(1:N,J),Y(1:N,K))
            DO,  I=1,N
               X(I,K) = X(I,K) - C*X(I,J)
            END DO
         END DO
C        2) Normalise column K
C        MATRIX PRODUCT Y = A*X, RESTRICTED TO COLUMN K OF MATRIX X
         CALL PROBTM(A,N,IBDA,LVBNDA,X(:,1:K),K,K,Y(:,1:K))
         C = DOT_PRODUCT(X(1:N,K),Y(1:N,K))
C         IF ( C .LT. 1.D-30) THEN
C            WRITE (*,1059) K, IP
C1059        FORMAT(' SIMVIT: MODE',I4,' CANNOT BE NORMALIZED. IP=',I4)
C            IER = -101
C            GOTO 999
C         END IF
         C = 1.D0/SQRT(C)
         DO,  I=1,N
            X(I,K) = C*X(I,K)
         END DO
      END DO
C
C     SOLVE EIGENVALUE PROBLEM IN THE SUBSPACE SPANNED BY COLUMNS OF
C     MATRIX X WHICH DID NOT CONVERGE YET.
      JAC = JAC + 1
C     REMAINING COLUMNS=DIMENSION OF SUBSPACE:
      NS = NEIGIT - KEM + 1
C
C     PROJECTION OF B INTO SUBSPACE. NOTE: PROJECTION OF A IS UNIT
C     MATRIX DUE TO ORTHO-NORMALISATION
C
C     MATRIX PRODUCT Y = B*X, WITH X FROM COLUMN KEM ON
      CALL PROBTM(B,N,IBDB,LVBNDB,X,KEM,NEIGIT,Y)
C
      J = 0
      DO,  K=KEM,NEIGIT
         J = J + 1
         I = 0
         DO,  L=KEM,K
            I  = I + 1
            IJ = LVTRI(J) + I
            BS(IJ) = DOT_PRODUCT(X(1:N,K),Y(1:N,L))
         END DO
      END DO
C     (BS: UPPER TRIANGULAR MATRIX STORED IN 1-DIM MODE)
C
C     SOLVE EIGENVALUE PROBLEM FOR BS BY JACOBI METHOD
C     RESULT:
C     BS <-- DIAGONAL ELEMENTS WILL CONTAIN EIGENVALUES
C     YS <-- MODAL MATRIX
C
      CALL JACOB2(BS,NS,LVTRI,EXACT,YS(:NS,:NS),IER)
      IF (IER .NE. 0) THEN
         WRITE (*,1060) JAC
         WRITE (*,1061) IER
      END IF
C
      K = KEM - 1
      DO,  I=1,NS
         K  = K + 1
         II = LVTRI(I) + I
         ELAM(K) = 1.D0/BS(II)
      END DO
C
C     DETERMINATION OF ARRAY LH INDICATING ORDER OF EIGENVALUES
      DO,  I=KEM,NEIGIT
         LH(I) = I
      END DO
      DO,  K=KEM,NITM1
         EGWR = ELAM(K)
         J    = K
         KP1  = K + 1
         DO,  I=KP1,NEIGIT
            IF (ABS(ELAM(I)) .GE. ABS(EGWR)) CYCLE
               EGWR = ELAM(I)
               J    = I
         END DO
         IF (J .NE. K) THEN
            ELAM(J) = ELAM(K)
            ELAM(K) = EGWR
            KK      = LH(K)
            LH(K)   = LH(J)
            LH(J)   = KK
         END IF
      END DO
      LH1(1:NS-1)=LH(KEM:NITM1)-KEM+1
C
C     CHECK FOR CONVERGENCE.
C     ANY OFF-DIAGONAL ELEMENT .GT. EPS IN A COLUMN OF YS  MEANS:
C     NO CONVERGENCE OF EIGENVECTOR CORRESPONDING TO THAT COLUMN
      KRV = KEM
      OUTER: DO,  JJ=1,NS-1
         J=LH1(JJ)
         INNER: DO,  I=1,NS
            IF (I .NE. J) THEN
               IF (ABS(YS(I,J)) .GT. EPS)  EXIT OUTER
            END IF
         END DO INNER
         KRV = KRV + 1
      END DO OUTER
C
C     CALCULATION OF IMPROVED APPROXIMATION OF NON-CONVERGED
C     EIGENVECTORS.
C     RATHER THAN COMPUTING X = X*YS AND Y = B*X, THE NEW COLUMNS OF
C     Y ARE COMPUTED:  Y= B*X = B*(X*YS) = (B*X)*YS = Y*YS .
C     THIS WILL SAVE MATRIX MULTIPLICATIONS.
      Y(:,KEM:NITM1)=MATMUL(Y(:,KEM:NEIGIT),YS(1:NS,LH1(1:NS-1)))
C
      IF (KRV .NE. KEM) THEN
C        CALCULATION OF CONVERGED EIGENVECTORS
         X(:,KEM:KRV-1)=
     *   MATMUL(X(:,KEM:NEIGIT),YS(1:NS,LH1(1:KRV-KEM)))
         KEM = KRV
      END IF
C
C     HAVE ALL REQUESTED EIGENMODES CONVERGED OR NOT?
      IF (KEM .LT. NEIGRQ+1) THEN
C        NOT ALL REQUESTED MODES HAVE CONVERGED.
C        ADJUST NUMBER M OF SIMPLE POWER ITERATION STEPS
C        AND BRANCH TO BEGIN OF PRINCIPLE LOOP
         PA = ABS(ELAM(1))**M
         PB = 1.D0/ABS(ELAM(NEIGRQ))**M
         IF (M .LT. 5) THEN
            IF ((PA.GT.1D-10 .AND. PB.GT.1D-10) .AND. PA*PB.GT.1D-4)
     *           M = M+1
         END IF
         IF (M .GT. 1 .AND. PA*PB .LT. 1.0D-8)   M = M - 1
         GOTO 90
      END IF
C
C     END OF PRINCIPAL LOOP
C***********************************************************************
C
  999 DEALLOCATE(Y,H,CH,BS, LH)
      RETURN
 1058 FORMAT (1X,'SIMVIT: ERROR IN CHOLESKY DECOMPOSITION. IER=',I4)
 1060 FORMAT(1X,'SIMVIT: EIGENVALUE PROBLEM IN SUBSPACE JAC=',I4)
 1061 FORMAT(1X,'COULD NOT BE SOLVED.JACOB2 ERROR CODE IS IER=',I4)
C     END OF SUBROUTINE SIMVIT
      END
