C     Last change:  BG   20 Sep 2004    1:03 pm
      SUBROUTINE TRNPSI(M,PSI,A,B,IER)
C
C     HEADER:    SUBROUTINE FOR STIFFNESS MATRIX TRANSFORMATION
C     CONTENTS:  GIVEN THE ANGLE BETWEEN TWO COORDINATE SYSTEMS,
C                THE MATRIX TRANSFORMATION IS PERFORMED.
C                B=T(TRANSPOSE)*A*T. T IS THE TRANSFORMATION MATRIX.
C                IF (M. EQ. 2) THEN
C                   TRANSFORMATION IS FOR TRANSVERSE SHEAR
C                IF (M .EQ. 3) THEN
C                   TRANSFORMATION IS FOR IN-PLANE, COUPLING
C                   OR BENDING STIFFNESS, OR FOR Q-MATRIX
C                ELSE
C                   SUBROUTINE IS NOT APPLICABLE
C                END IF
C                THE SYSTEM IN WHICH MATRIX A IS DEFINED
C                IS ROTATED COUNTER-CLOCKWISE BY THE ANGLE PSI
C                FROM THE SYSTEM OF THE MATRIX B.
C     INPUT:
C
C         A(M,M)  - THE MATRIX TO BE TRANSFORMED.
C
C             M   - THE RANK OF THE TRANSFORMATION MATRIX T.
C
C           PSI   - THE ANGLE BETWEEN TWO COORDINATE SYSTEMS (DEGREES)
C
C     RETURN:
C
C         B(M,M)  - STORING THE TRANSFORMED MATRIX A.
C         IER     - ERROR INDICATOR:
C                   IER = 0: NO ERROR ENCOUNTERED
C                   IER = 1: ARGUMENT M IS NEITHER 2 NOR 3
C                   IER = 2: ERROR ENCOUNTERED WHEN TRANSFORMING
C                            TRANSVERSE SHEAR STIFFNESS (2,2)
C                   IER = 3: ERROR ENCOUNTERED WHEN TRANSFORMING
C                            (3,3)-MATRIX, E.G. Q-MATRIX
C
C     COMMON:
C
C
C     CALLS: MULTI3
C
C     CALLED BY: ABDHL
C
C     AUTHOR:    Dr. B.GEIER, DLR/WB-SM
C                J. YUAN , DLR/WB-SM, 17.11.89 , REV.  28.02.92
C                B. GEIER, REV. 14.09.92
C --------------------------------------------------------------------+
C      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      IMPLICIT NONE
      INTEGER,INTENT(OUT):: IER
      INTEGER,INTENT(IN)::  M
      REAL,INTENT(IN):: PSI
      REAL,DIMENSION(1:M,1:M),INTENT(IN)::  A
      REAL,DIMENSION(1:M,1:M),INTENT(OUT):: B
      REAL:: PAI,C,CC,S,SS,CS,PHI
      REAL,DIMENSION(1:3,1:3):: T
      REAL,DIMENSION(1:2,1:2):: T1
C
      PARAMETER (PAI=3.141592654/180.)
C      DIMENSION A(M,M),B(M,M)
C      DIMENSION X1(2,2),X2(2,2),T1(2,2),Y1(3,3),Y2(3,3),T(3,3)
C      DIMENSION T1(2,2),T(3,3)
      PHI=PSI*PAI
C      DU = 1.D0
      S=SIN(PHI)
      C=COS(PHI)
      IER=0
      IF (M.EQ.2) THEN
         T1(1,1)=C
         T1(2,2)=C
         T1(1,2)=S
         T1(2,1)=-S
C         CALL MULTI3(2,T1,2,2,A,2,2,X1,2,2,B,2,2,X2,DU,IER)
C            IF (IER.NE.0) THEN
C               IER=2
C            ENDIF
         B=MATMUL(TRANSPOSE(T1),MATMUL(A,T1))
      ELSE IF (M.EQ.3) THEN
         CC=C*C
         SS=S*S
         CS=C*S
         T(1,1)=CC
         T(2,2)=CC
         T(3,3)=CC-SS
         T(1,2)=SS
         T(2,1)=SS
         T(1,3)=CS
         T(3,1)=-2.0*CS
         T(2,3)=-CS
         T(3,2)=2.0*CS
C         CALL MULTI3(2,T,3,3,A,3,3,Y1,3,3,B,3,3,Y2,DU,IER)
C            IF (IER.NE.0) THEN
C                IER=3
C             ENDIF
         B=MATMUL(TRANSPOSE(T),MATMUL(A,T))
      ELSE
         IER = 1
      ENDIF
      IF (IER .NE. 0) WRITE(*,1000) IER
      RETURN
1000  FORMAT(2X,'ERROR NO. ',I2,' IN TRNPSI')
C     END OF SUBROUTINE TRNPSI
      END
