

import sys, os
import subprocess
import shutil
import re
import getpass

#===============================================================================
# adapt paths if necessary
#===============================================================================
delisPath = None
pathToPython=None
if 'win' in sys.platform:
    if getpass. getuser() == 'freu_se':
        delisPath = 'D:/freu_se/DELiS'
    elif getpass. getuser() == 'schu_a1':
        delisPath = 'D:/schu_a1/DELiS_full/trunk'
        
    possiblePathsToPython = ['C:/Program Files/Miniconda3/envs/py35', 'C:/Miniconda3/envs/py35', 'C:/Miniconda3/envs/Python35', 'C:/Python35']
    for possiblePath in possiblePathsToPython:
        if not os.path.exists(possiblePath):
            pathToPython = possiblePath
            break
    if pathToPython is None:
        raise Exception('Could not locate python executable')
elif 'linux' in sys.platform:
    pathToPython = '/cluster/software/python3.5/'

#===============================================================================
# end adapt paths
#===============================================================================

if not os.path.exists('BEOSMain.for'):
    raise Exception('not in beos folder')

def overwriteDiscretizationInBeos(discretization):
    """doc"""
    # script to automatically overwrite P_RTNDS parameter within #prams file
    # useful when BEOS needs to be recompiled
    # the script gets one parameter via a console-based execution which
    # represents the new value of the P_RTNDS parameter within the #prams file
    
    # Read #prams file
    fileStream = open("#PRAMS","r")
    lines = fileStream.read()
    fileStream.close()
    
    #Change #prams file with new P_RTNDS parameter
    pattern = re.compile("P_RTNDS=\d*,")
    newLines = pattern.sub("P_RTNDS=%s," % str(discretization),  lines)
    
    #Overwrite old file
    with open("#PRAMS","w") as f:
        f.write(newLines)

def createLib():
    """doc"""
    programName = 'beos'
    discretizationsToBuild = [19, 21,
                              25,31]
    if 'win' in sys.platform:
        pythonExe = os.path.join(pathToPython, 'python.exe')
        f2pyScript = os.path.join(pathToPython, "Scripts/f2py.py")
        compilerArgString = '-c --compiler=msvc --fcompiler=intelvem'
        beosLib=programName+'.cp35-win_amd64.pyd'
    elif 'linux' in sys.platform:
        # intel fortran is not working with beos! Beos starts and hangs when trying to calculate the eigenvalues
    #     # source fortran compiler args
    #     subprocess.call(['source', '/cluster/software/intel/composerxe/bin/compilervars.sh', 'intel64'])
        #compilerArgString = '-c --fcompiler=intelem'
        compilerArgString = '-c --fcompiler=gnu95'
        pythonExe = os.path.join(pathToPython, 'bin/python3.5')
        f2pyScript = os.path.join(pathToPython, "bin/f2py")
        beosLib=programName+'.cpython-35m-x86_64-linux-gnu.so'
        
        
    f2pyCall = [pythonExe, f2pyScript]
    if os.path.exists(beosLib):
        os.remove(beosLib)
        
    beosFiles = 'Main.f90 BEOSMain.for A11xst.for A11yst.for A21ast.for A22ast.for ABDetc.for BmtstN.for BuVi.for Chneig.for Chodec.for Chosol.for Cktype.for Cmpa11.for Cmpa21.for Cmpa22.for Cmpbmt.for Cmpmmt.for Cntr.for CripplingO.for CripplingT.for EIGVPR.FOR F01kbf.for F01KTF.FOR GlBuck.for Incs2.for Incs3N.for Iuoriv3.for JACOB2.FOR Kuorkv2.for Lqfit95.for MKLVBD.F90 Mmtst.for Modes.for Phi.for Plotmesh.for PPROBTM95.for Prtout.for Prtvib.for PRTYPE.FOR RddataN-Ak.for Rdtype.for Redfkq.for Setkey_2.for SIMVT2003.FOR TCZINC.FOR Tfmstf.for TMINC.FOR Trnpsi.for Vibs2.for Vibs3NN.for'
        
    compilerArgString += ' -m %s %s' % (programName, beosFiles)
    for discretization in discretizationsToBuild:
        print('==================================')
        print('Build BEOS with value '+str(discretization))
        print('==================================')

        if delisPath is None:
            delisBeosLibPath = os.path.expanduser("prtnds" + str(discretization))
        else:
            delisBeosLibPath = os.path.join(delisPath, 'src', 'buckling', 'beoslib', 'prtnds'+str(discretization))
        if not os.path.exists(delisBeosLibPath):
            os.makedirs(delisBeosLibPath)
            with open(os.path.join(delisBeosLibPath, '__init__.py'), 'w'):
                pass
        beosTargetFile = os.path.join(delisBeosLibPath, beosLib)
        if os.path.exists(beosTargetFile):
            os.remove(beosTargetFile)
    
        overwriteDiscretizationInBeos(discretization)
        command = f2pyCall + compilerArgString.split()
        print('call f2py with this command: {}'.format(' '.join(command)))
        subprocess.call(command)
    
        print('copy to: '+delisBeosLibPath)
        shutil.copy(beosLib, delisBeosLibPath)

    print('==================================')
    print('Finished')
    print('==================================')









if __name__ == '__main__':
    createLib()






